@@ -67,38 +67,32 @@ typedef struct {
     uint8_t sak;
 } PACKED card_clone_t;
 
-int get_block_count(iso14a_card_select_t card, uint8_t *version, uint16_t version_len);
-uint16_t get_ev1_version(iso14a_card_select_t card, uint8_t *version, uint16_t version_len);
-uint16_t get_ev1_signature(iso14a_card_select_t card, uint8_t *signature, uint16_t sign_len);
-uint16_t get_ev1_counter(iso14a_card_select_t card, uint8_t counter, uint8_t *response, uint16_t resp_len);
-uint16_t get_ev1_tearing(iso14a_card_select_t card, uint8_t counter, uint8_t *response, uint16_t resp_len);
-
-uint16_t get_ev1_version(iso14a_card_select_t card, uint8_t *version, uint16_t version_len) {
+static uint16_t get_ev1_version(iso14a_card_select_t card, uint8_t *version, uint16_t version_len) {
     return mifare_sendcmd(MIFARE_ULEV1_VERSION, NULL, 0, version, version_len, NULL, NULL);
 }
 
-uint16_t get_ev1_signature(iso14a_card_select_t card, uint8_t *signature, uint16_t sign_len) {
+static uint16_t get_ev1_signature(iso14a_card_select_t card, uint8_t *signature, uint16_t sign_len) {
     uint8_t cmd[4] = {MIFARE_ULEV1_READSIG, 0x00, 0x00, 0x00};
     AddCrc14A(cmd, 2);
     ReaderTransmit(cmd, sizeof(cmd), NULL);
     return ReaderReceive(signature, sign_len, NULL);
 }
 
-uint16_t get_ev1_counter(iso14a_card_select_t card, uint8_t counter, uint8_t *response, uint16_t resp_len) {
+static uint16_t get_ev1_counter(iso14a_card_select_t card, uint8_t counter, uint8_t *response, uint16_t resp_len) {
     uint8_t cmd[4] = {MIFARE_ULEV1_READ_CNT, counter, 0x00, 0x00};
     AddCrc14A(cmd, 2);
     ReaderTransmit(cmd, sizeof(cmd), NULL);
     return ReaderReceive(response, resp_len, NULL);
 }
 
-uint16_t get_ev1_tearing(iso14a_card_select_t card, uint8_t counter, uint8_t *response, uint16_t resp_len) {
+static uint16_t get_ev1_tearing(iso14a_card_select_t card, uint8_t counter, uint8_t *response, uint16_t resp_len) {
     uint8_t cmd[4] = {MIFARE_ULEV1_CHECKTEAR, counter, 0x00, 0x00};
     AddCrc14A(cmd, 2);
     ReaderTransmit(cmd, sizeof(cmd), NULL);
     return ReaderReceive(response, resp_len, NULL);
 }
 
-int get_block_count(iso14a_card_select_t card, uint8_t *version, uint16_t version_len) {
+static int get_block_count(iso14a_card_select_t card, const uint8_t *version, uint16_t version_len) {
     // Default to MAX_DEFAULT_BLOCKS blocks
     int block_count = MAX_DEFAULT_BLOCKS;
     // Most of this code is from cmdhfmfu.c
@@ -248,10 +242,11 @@ void RunMod(void) {
                     state = STATE_SEARCH;
                 }
             } else if (state == STATE_EMUL) {
-                uint16_t flags = FLAG_7B_UID_IN_DATA;
+                uint16_t flags = 0;
+                FLAG_SET_UID_IN_DATA(flags, 7);
 
                 Dbprintf("Starting simulation, press " _GREEN_("pm3 button") " to stop and go back to search state.");
-                SimulateIso14443aTag(7, flags, card.uid, 0);
+                SimulateIso14443aTag(7, flags, card.uid, 0, NULL, 0);
 
                 // Go back to search state if user presses pm3-button
                 state = STATE_SEARCH;
