@@ -1,9 +1,18 @@
 //-----------------------------------------------------------------------------
-// tharexde, 2021
+// Copyright (C) tharexde, 2021
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
 //
-// This code is licensed to you under the terms of the GNU GPL, version 2 or,
-// at your option, any later version. See the LICENSE.txt file for the text of
-// the license.
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
 //-----------------------------------------------------------------------------
 // main code for EM4x50 simulator and collector aka THAREXDE
 //-----------------------------------------------------------------------------
@@ -219,7 +228,7 @@ void RunMod(void) {
                 SpinDelay(200);
             }
 
-            em4x50_handle_commands(&command, tag);
+            em4x50_handle_commands(&command, tag, true);
 
             // check if new password was found
             if (g_Password != reflect32(tag[EM4X50_DEVICE_PASSWORD])) {
