@@ -46,6 +46,11 @@ int flashmem_spiffs_load(const char *destfn, const uint8_t *data, size_t datalen
         uint32_t bytes_in_packet = MIN(FLASH_MEM_BLOCK_SIZE, bytes_remaining);
 
         flashmem_write_t *payload = calloc(1, sizeof(flashmem_write_t) + bytes_in_packet);
+        if (payload == NULL) {
+            PrintAndLogEx(WARNING, "Failed to allocate memory");
+            ret_val = PM3_EMALLOC;
+            goto out;
+        }
 
         payload->append = (bytes_sent > 0);
 
@@ -69,7 +74,7 @@ int flashmem_spiffs_load(const char *destfn, const uint8_t *data, size_t datalen
 
         uint8_t retry = 3;
         while (WaitForResponseTimeout(CMD_SPIFFS_WRITE, &resp, 2000) == false) {
-            PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+            PrintAndLogEx(WARNING, "timeout while waiting for reply");
             retry--;
             if (retry == 0) {
                 ret_val = PM3_ETIMEOUT;
@@ -96,7 +101,7 @@ int flashmem_spiffs_download(char *fn, uint8_t fnlen, void **pdest, size_t *dest
     SendCommandNG(CMD_SPIFFS_STAT, (uint8_t *)fn, fnlen);
     PacketResponseNG resp;
     if (WaitForResponseTimeout(CMD_SPIFFS_STAT, &resp, 2000) == false) {
-        PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+        PrintAndLogEx(WARNING, "timeout while waiting for reply");
         return PM3_ETIMEOUT;
     }
 
@@ -108,7 +113,7 @@ int flashmem_spiffs_download(char *fn, uint8_t fnlen, void **pdest, size_t *dest
 
     *pdest = calloc(len, sizeof(uint8_t));
     if (*pdest == false) {
-        PrintAndLogEx(ERR, "error, cannot allocate memory ");
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return PM3_EMALLOC;
     }
 
@@ -262,9 +267,9 @@ static int CmdFlashMemSpiFFSRemove(const char *Cmd) {
     clearCommandBuffer();
     SendCommandNG(CMD_SPIFFS_REMOVE, (uint8_t *)&payload, sizeof(payload));
     WaitForResponse(CMD_SPIFFS_REMOVE, &resp);
-    if (resp.status == PM3_SUCCESS)
+    if (resp.status == PM3_SUCCESS) {
         PrintAndLogEx(INFO, "Done!");
-
+    }
     return PM3_SUCCESS;
 }
 
@@ -310,10 +315,11 @@ static int CmdFlashMemSpiFFSRename(const char *Cmd) {
     clearCommandBuffer();
     SendCommandNG(CMD_SPIFFS_RENAME, (uint8_t *)&payload, sizeof(payload));
     WaitForResponse(CMD_SPIFFS_RENAME, &resp);
-    if (resp.status == PM3_SUCCESS)
+    if (resp.status == PM3_SUCCESS) {
         PrintAndLogEx(INFO, "Done!");
+    }
 
-    PrintAndLogEx(HINT, "Try `" _YELLOW_("mem spiffs tree") "` to verify");
+    PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("mem spiffs tree") "` to verify");
     return PM3_SUCCESS;
 }
 
@@ -358,10 +364,11 @@ static int CmdFlashMemSpiFFSCopy(const char *Cmd) {
     clearCommandBuffer();
     SendCommandNG(CMD_SPIFFS_COPY, (uint8_t *)&payload, sizeof(payload));
     WaitForResponse(CMD_SPIFFS_COPY, &resp);
-    if (resp.status == PM3_SUCCESS)
+    if (resp.status == PM3_SUCCESS) {
         PrintAndLogEx(INFO, "Done!");
+    }
 
-    PrintAndLogEx(HINT, "Try `" _YELLOW_("mem spiffs tree") "` to verify");
+    PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("mem spiffs tree") "` to verify");
     return PM3_SUCCESS;
 }
 
@@ -399,14 +406,14 @@ static int CmdFlashMemSpiFFSDump(const char *Cmd) {
     SendCommandNG(CMD_SPIFFS_STAT, (uint8_t *)src, slen);
     PacketResponseNG resp;
     if (WaitForResponseTimeout(CMD_SPIFFS_STAT, &resp, 2000) == false) {
-        PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+        PrintAndLogEx(WARNING, "timeout while waiting for reply");
         return PM3_ETIMEOUT;
     }
 
     uint32_t len = resp.data.asDwords[0];
     uint8_t *dump = calloc(len, sizeof(uint8_t));
     if (dump == NULL) {
-        PrintAndLogEx(ERR, "error, cannot allocate memory ");
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return PM3_EMALLOC;
     }
 
@@ -426,7 +433,7 @@ static int CmdFlashMemSpiFFSDump(const char *Cmd) {
             free(dump);
             return PM3_EMALLOC;
         }
-        PrintAndLogEx(HINT, "Use 'trace list -1 -t ...' to view, 'trace save -f ...' to save");
+        PrintAndLogEx(HINT, "Hint: Use 'trace list -1 -t ...' to view, 'trace save -f ...' to save");
     }
 
 
@@ -474,10 +481,11 @@ static int CmdFlashMemSpiFFSWipe(const char *Cmd) {
     clearCommandBuffer();
     SendCommandNG(CMD_SPIFFS_WIPE, NULL, 0);
     WaitForResponse(CMD_SPIFFS_WIPE, &resp);
-    if (resp.status == PM3_SUCCESS)
+    if (resp.status == PM3_SUCCESS) {
         PrintAndLogEx(INFO, "Done!");
+    }
 
-    PrintAndLogEx(HINT, "Try `" _YELLOW_("mem spiffs tree") "` to verify");
+    PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("mem spiffs tree") "` to verify");
     return PM3_SUCCESS;
 }
 
@@ -525,7 +533,7 @@ static int CmdFlashMemSpiFFSUpload(const char *Cmd) {
     if (res == PM3_SUCCESS)
         PrintAndLogEx(SUCCESS, "Wrote "_GREEN_("%zu") " bytes to file "_GREEN_("%s"), datalen, dest);
 
-    PrintAndLogEx(HINT, "Try `" _YELLOW_("mem spiffs tree") "` to verify");
+    PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("mem spiffs tree") "` to verify");
     return res;
 }
 
@@ -568,10 +576,11 @@ static int CmdFlashMemSpiFFSView(const char *Cmd) {
 
 static command_t CommandTable[] = {
     {"help",    CmdHelp,                  AlwaysAvailable, "This help"},
+    {"-----------", CmdHelp,                  IfPm3Flash,      "------------------- " _CYAN_("Operations") " -------------------"},
     {"copy",    CmdFlashMemSpiFFSCopy,    IfPm3Flash, "Copy a file to another (destructively) in SPIFFS file system"},
     {"check",   CmdFlashMemSpiFFSCheck,   IfPm3Flash, "Check/try to defrag faulty/fragmented file system"},
     {"dump",    CmdFlashMemSpiFFSDump,    IfPm3Flash, "Dump a file from SPIFFS file system"},
-    {"info",    CmdFlashMemSpiFFSInfo,    IfPm3Flash, "Print file system info and usage statistics"},
+    {"info",        CmdFlashMemSpiFFSInfo,    IfPm3Flash,      "File system information and usage statistics"},
     {"mount",   CmdFlashMemSpiFFSMount,   IfPm3Flash, "Mount the SPIFFS file system if not already mounted"},
     {"remove",  CmdFlashMemSpiFFSRemove,  IfPm3Flash, "Remove a file from SPIFFS file system"},
     {"rename",  CmdFlashMemSpiFFSRename,  IfPm3Flash, "Rename/move a file in SPIFFS file system"},
