@@ -39,7 +39,7 @@
 // Average EF_DG2 seems to be around 20-25kB or so, but ICAO doesn't set an upper limit
 // Iris data seems to be suggested to be around 35kB per eye (Presumably bumping up the file size to around 70kB)
 // but as we cannot read that until we implement PACE, 35k seems to be a safe point.
-#define EMRTD_MAX_FILE_SIZE 35000
+#define EMRTD_MAX_FILE_SIZE 70000
 
 // ISO7816 commands
 #define EMRTD_P1_SELECT_BY_EF       0x02
@@ -411,13 +411,13 @@ static void emrtd_deskey(uint8_t *seed, const uint8_t *type, int length, uint8_t
     PrintAndLogEx(DEBUG, "seed.............. %s", sprint_hex_inrow(seed, 16));
 
     // combine seed and type
-    uint8_t data[50];
+    uint8_t data[50] = { 0x00 };
     memcpy(data, seed, length);
     memcpy(data + length, type, 4);
     PrintAndLogEx(DEBUG, "data.............. %s", sprint_hex_inrow(data, length + 4));
 
     // SHA1 the key
-    unsigned char key[64];
+    unsigned char key[64] = { 0x00 };
     sha1hash(data, length + 4, key);
     PrintAndLogEx(DEBUG, "key............... %s", sprint_hex_inrow(key, length + 4));
 
@@ -767,37 +767,51 @@ static bool emrtd_select_and_read(uint8_t *dataout, size_t *dataoutlen, uint16_t
 
 static const uint8_t jpeg_header[4] = { 0xFF, 0xD8, 0xFF, 0xE0 };
 static const uint8_t jpeg2k_header[6] = { 0x00, 0x00, 0x00, 0x0C, 0x6A, 0x50 };
+static const uint8_t jpeg2k_cs_header[4] = { 0xFF, 0x4F, 0xFF, 0x51 };
 
 static int emrtd_dump_ef_dg2(uint8_t *file_contents, size_t file_length, const char *path) {
     size_t offset;
     int datalen = 0;
+    char suffix[5] = { '\0' };
 
     // This is a hacky impl that just looks for the image header. I'll improve it eventually.
     // based on mrpkey.py
     // Note: Doing file_length - 6 to account for the longest data we're checking.
     // Checks first byte before the rest to reduce overhead
     for (offset = 0; offset < file_length - 6; offset++) {
-        if ((file_contents[offset] == 0xFF && memcmp(jpeg_header, file_contents + offset, 4) == 0) ||
-                (file_contents[offset] == 0x00 && memcmp(jpeg2k_header, file_contents + offset, 6) == 0)) {
+        if (file_contents[offset] == 0xFF) {
+            if (memcmp(jpeg_header, file_contents + offset, 4) == 0) {
+                datalen = file_length - offset;
+                strcpy(suffix, ".jpg");
+                break;
+            } else if (memcmp(jpeg2k_cs_header, file_contents + offset, 4) == 0) {
+                datalen = file_length - offset;
+                // no standardized extension for codestream data, using .jpc
+                strcpy(suffix, ".jpc");
+                break;
+            }
+        } else if (file_contents[offset] == 0x00 && memcmp(jpeg2k_header, file_contents + offset, 6) == 0) {
+            strcpy(suffix, ".jp2");
             datalen = file_length - offset;
             break;
         }
     }
-
     // If we didn't get any data, return false.
     if (datalen == 0) {
         return PM3_ESOFT;
     }
 
     char *filepath = calloc(strlen(path) + 100, sizeof(char));
-    if (filepath == NULL)
+    if (filepath == NULL) {
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return PM3_EMALLOC;
+    }
 
     strcpy(filepath, path);
     strncat(filepath, PATHSEP, 2);
     strcat(filepath, dg_table[EF_DG2].filename);
 
-    saveFile(filepath, file_contents[offset] == 0xFF ? ".jpg" : ".jp2", file_contents + offset, datalen);
+    saveFile(filepath, suffix, file_contents + offset, datalen);
 
     free(filepath);
     return PM3_SUCCESS;
@@ -815,6 +829,7 @@ static int emrtd_dump_ef_dg5(uint8_t *file_contents, size_t file_length, const c
     if (datalen < EMRTD_MAX_FILE_SIZE) {
         char *filepath = calloc(strlen(path) + 100, sizeof(char));
         if (filepath == NULL) {
+            PrintAndLogEx(WARNING, "Failed to allocate memory");
             return PM3_EMALLOC;
         }
         strcpy(filepath, path);
@@ -843,6 +858,7 @@ static int emrtd_dump_ef_dg7(uint8_t *file_contents, size_t file_length, const c
     if (datalen < EMRTD_MAX_FILE_SIZE) {
         char *filepath = calloc(strlen(path) + 100, sizeof(char));
         if (filepath == NULL) {
+            PrintAndLogEx(WARNING, "Failed to allocate memory");
             return PM3_EMALLOC;
         }
         strcpy(filepath, path);
@@ -870,6 +886,7 @@ static int emrtd_dump_ef_sod(uint8_t *file_contents, size_t file_length, const c
 
     char *filepath = calloc(strlen(path) + 100, sizeof(char));
     if (filepath == NULL) {
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return PM3_EMALLOC;
     }
 
@@ -892,6 +909,7 @@ static bool emrtd_dump_file(uint8_t *ks_enc, uint8_t *ks_mac, uint8_t *ssc, uint
 
     char *filepath = calloc(strlen(path) + 100, sizeof(char));
     if (filepath == NULL) {
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return false;
     }
 
@@ -945,7 +963,7 @@ static bool emrtd_do_bac(char *documentnumber, char *dob, char *expiry, uint8_t
     char dobcd = emrtd_calculate_check_digit(dob);
     char expirycd = emrtd_calculate_check_digit(expiry);
 
-    char kmrz[25];
+    char kmrz[25] = { 0x00 };
     snprintf(kmrz, sizeof(kmrz), "%s%i%s%i%s%i", documentnumber, documentnumbercd, dob, dobcd, expiry, expirycd);
     PrintAndLogEx(DEBUG, "kmrz.............. " _GREEN_("%s"), kmrz);
 
@@ -1065,7 +1083,7 @@ static bool emrtd_do_auth(char *documentnumber, char *dob, char *expiry, bool BA
         // If BAC isn't available, exit out and warn user.
         if (BAC_available == false) {
             PrintAndLogEx(ERR, "This eMRTD enforces authentication, but you didn't supply MRZ data. Cannot proceed.");
-            PrintAndLogEx(HINT, "Check out `hf emrtd info/dump --h`, supply data with `-n` `-d` and `-e`");
+            PrintAndLogEx(HINT, "Hint: Check out `" _YELLOW_("hf emrtd info/dump --h") "`, supply data with `-n` `-d` and `-e`");
             return false;
         }
 
@@ -1093,7 +1111,7 @@ int dumpHF_EMRTD(char *documentnumber, char *dob, char *expiry, bool BAC_availab
     // Dump EF_CardAccess (if available)
     if (emrtd_dump_file(ks_enc, ks_mac, ssc, dg_table[EF_CardAccess].fileid, dg_table[EF_CardAccess].filename, BAC, path) == false) {
         PrintAndLogEx(INFO, "Couldn't dump EF_CardAccess, card does not support PACE");
-        PrintAndLogEx(HINT, "This is expected behavior for cards without PACE, and isn't something to be worried about");
+        PrintAndLogEx(HINT, "Hint: This is expected behavior for cards without PACE and isn't something to be worried about");
     }
 
     // Authenticate with the eMRTD
@@ -1112,6 +1130,7 @@ int dumpHF_EMRTD(char *documentnumber, char *dob, char *expiry, bool BAC_availab
 
     char *filepath = calloc(strlen(path) + 100, sizeof(char));
     if (filepath == NULL) {
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return PM3_EMALLOC;
     }
 
@@ -1377,7 +1396,7 @@ static void emrtd_print_unknown_timestamp_5f85(uint8_t *data, size_t datalen) {
             );
 
     PrintAndLogEx(SUCCESS, "Unknown timestamp 5F85... " _YELLOW_("%s"), final_date);
-    PrintAndLogEx(HINT, "This is very likely the personalization timestamp, but it is using an undocumented tag.");
+    PrintAndLogEx(HINT, "Hint: This is very likely the personalization timestamp but it is using an undocumented tag.");
 }
 
 static int emrtd_print_ef_com_info(uint8_t *data, size_t datalen) {
@@ -1399,7 +1418,8 @@ static int emrtd_print_ef_com_info(uint8_t *data, size_t datalen) {
             continue;
         }
         int n = 25 - strlen(dg->filename);
-        PrintAndLogEx(SUCCESS, "%s%*.*s " _YELLOW_("%s"), dg->filename, n, n, pad, dg->desc);
+        PrintAndLogEx(SUCCESS, "%s%.*s " _YELLOW_("%s"), dg->filename, n, pad, dg->desc);
+
     }
     return PM3_SUCCESS;
 }
@@ -1862,7 +1882,7 @@ static int emrtd_parse_ef_sod_hashes(uint8_t *data, size_t datalen, uint8_t *has
 static int emrtd_print_ef_sod_info(uint8_t *dg_hashes_calc, uint8_t *dg_hashes_sod, int hash_algo, bool fastdump) {
     PrintAndLogEx(NORMAL, "");
     PrintAndLogEx(INFO, "------------------------ " _CYAN_("EF_SOD") " ------------------------");
-    PrintAndLogEx(INFO, "Document Security Object, ");
+    PrintAndLogEx(INFO, "Document Security Object");
     PrintAndLogEx(INFO, "contains the digital signatures of the passport data");
     PrintAndLogEx(INFO, "");
 
@@ -1891,17 +1911,17 @@ static int emrtd_print_ef_sod_info(uint8_t *dg_hashes_calc, uint8_t *dg_hashes_s
             if (calc_all_zero == true) {
 
                 if (fastdump && !dg_table[i].fastdump && !dg_table[i].pace && !dg_table[i].eac) {
-                    PrintAndLogEx(SUCCESS, "EF_DG%2i %s %*.*s File was skipped, but is in EF_SOD", i, dg_table[i].desc, n, n, pad);
+                    PrintAndLogEx(SUCCESS, _YELLOW_("EF_DG%-2i") " %s%.*s File was skipped, but is in EF_SOD", i, dg_table[i].desc, n, pad);
                 } else {
-                    PrintAndLogEx(SUCCESS, "EF_DG%2i %s %*.*s File couldn't be read, but is in EF_SOD", i, dg_table[i].desc, n, n, pad);
+                    PrintAndLogEx(SUCCESS, _YELLOW_("EF_DG%-2i") " %s%.*s File couldn't be read, but is in EF_SOD", i, dg_table[i].desc, n, pad);
                 }
 
             } else if (sod_all_zero == true) {
-                PrintAndLogEx(SUCCESS, "EF_DG%2i %s %*.*s " _RED_("File is not in EF_SOD"), i, dg_table[i].desc, n, n, pad);
+                PrintAndLogEx(SUCCESS, _YELLOW_("EF_DG%-2i") " %s%.*s " _RED_("File is not in EF_SOD"), i, dg_table[i].desc, n, pad);
             } else if (hash_matches == false) {
-                PrintAndLogEx(SUCCESS, "EF_DG%2i %s %*.*s " _RED_("Invalid"), i, dg_table[i].desc, n, n, pad);
+                PrintAndLogEx(SUCCESS, _YELLOW_("EF_DG%-2i") " %s%.*s " _RED_("Invalid"), i, dg_table[i].desc, n, pad);
             } else {
-                PrintAndLogEx(SUCCESS, "EF_DG%2i %s %*.*s " _GREEN_("Valid"), i, dg_table[i].desc, n, n, pad);
+                PrintAndLogEx(SUCCESS, _YELLOW_("EF_DG%-2i") " %s%.*s " _GREEN_("Valid"), i, dg_table[i].desc, n, pad);
             }
         }
     }
@@ -1986,7 +2006,7 @@ int infoHF_EMRTD(char *documentnumber, char *dob, char *expiry, bool BAC_availab
     // Read EF_CardAccess
     if (emrtd_select_and_read(response, &resplen, dg_table[EF_CardAccess].fileid, ks_enc, ks_mac, ssc, BAC) == false) {
         PACE_available = false;
-        PrintAndLogEx(HINT, "The error above this is normal. It just means that your eMRTD lacks PACE.");
+        PrintAndLogEx(HINT, "Hint: The error above this is normal. It just means that your eMRTD lacks PACE.");
     }
 
     // Select and authenticate with the eMRTD
@@ -2073,7 +2093,7 @@ int infoHF_EMRTD(char *documentnumber, char *dob, char *expiry, bool BAC_availab
     DropField();
 
     emrtd_print_ef_sod_info(*dg_hashes_calc, *dg_hashes_sod, hash_algo, true);
-
+    PrintAndLogEx(NORMAL, "");
     return PM3_SUCCESS;
 }
 
@@ -2082,6 +2102,7 @@ int infoHF_EMRTD_offline(const char *path) {
     size_t datalen = 0;
     char *filepath = calloc(strlen(path) + 100, sizeof(char));
     if (filepath == NULL) {
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return PM3_EMALLOC;
     }
     strcpy(filepath, path);
@@ -2127,7 +2148,7 @@ int infoHF_EMRTD_offline(const char *path) {
         emrtd_print_ef_cardaccess_info(data, datalen);
         free(data);
     } else {
-        PrintAndLogEx(HINT, "The error above this is normal. It just means that your eMRTD lacks PACE");
+        PrintAndLogEx(HINT, "Hint: The error above this is normal. It just means that your eMRTD lacks PACE");
     }
 
     strcpy(filepath, path);
@@ -2155,11 +2176,13 @@ int infoHF_EMRTD_offline(const char *path) {
 
     // Read files in the file list
     for (int i = 0; i < filelistlen; i++) {
+
         emrtd_dg_t *dg = emrtd_tag_to_dg(filelist[i]);
         if (dg == NULL) {
             PrintAndLogEx(INFO, "File tag not found, skipping... %02X", filelist[i]);
             continue;
         }
+
         if (!dg->pace && !dg->eac) {
             strcpy(filepath, path);
             strncat(filepath, PATHSEP, 2);
@@ -2185,7 +2208,7 @@ int infoHF_EMRTD_offline(const char *path) {
     free(filepath);
 
     emrtd_print_ef_sod_info(*dg_hashes_calc, *dg_hashes_sod, hash_algo, false);
-
+    PrintAndLogEx(NORMAL, "");
     return PM3_SUCCESS;
 }
 
@@ -2248,7 +2271,7 @@ static int CmdHFeMRTDDump(const char *Cmd) {
     } else {
         if (!validate_date(dob, slen)) {
             PrintAndLogEx(ERR, "Date of birth date format is incorrect, cannot continue.");
-            PrintAndLogEx(HINT, "Use the format YYMMDD.");
+            PrintAndLogEx(HINT, "Hint: Use the format YYMMDD.");
             error = true;
         }
     }
@@ -2258,7 +2281,7 @@ static int CmdHFeMRTDDump(const char *Cmd) {
     } else {
         if (!validate_date(expiry, slen)) {
             PrintAndLogEx(ERR, "Expiry date format is incorrect, cannot continue.");
-            PrintAndLogEx(HINT, "Use the format YYMMDD.");
+            PrintAndLogEx(HINT, "Hint: Use the format YYMMDD.");
             error = true;
         }
     }
@@ -2276,12 +2299,12 @@ static int CmdHFeMRTDDump(const char *Cmd) {
             // TODO check MRZ checksums?
             if (!validate_date(dob, 6)) {
                 PrintAndLogEx(ERR, "Date of birth date format is incorrect, cannot continue.");
-                PrintAndLogEx(HINT, "Use the format YYMMDD.");
+                PrintAndLogEx(HINT, "Hint: Use the format YYMMDD.");
                 error = true;
             }
             if (!validate_date(expiry, 6)) {
                 PrintAndLogEx(ERR, "Expiry date format is incorrect, cannot continue.");
-                PrintAndLogEx(HINT, "Use the format YYMMDD.");
+                PrintAndLogEx(HINT, "Hint: Use the format YYMMDD.");
                 error = true;
             }
         }
@@ -2355,7 +2378,7 @@ static int CmdHFeMRTDInfo(const char *Cmd) {
     } else {
         if (!validate_date(dob, slen)) {
             PrintAndLogEx(ERR, "Date of birth date format is incorrect, cannot continue.");
-            PrintAndLogEx(HINT, "Use the format YYMMDD.");
+            PrintAndLogEx(HINT, "Hint: Use the format YYMMDD.");
             error = true;
         }
     }
@@ -2365,7 +2388,7 @@ static int CmdHFeMRTDInfo(const char *Cmd) {
     } else {
         if (!validate_date(expiry, slen)) {
             PrintAndLogEx(ERR, "Expiry date format is incorrect, cannot continue.");
-            PrintAndLogEx(HINT, "Use the format YYMMDD.");
+            PrintAndLogEx(HINT, "Hint: Use the format YYMMDD.");
             error = true;
         }
     }
@@ -2383,12 +2406,12 @@ static int CmdHFeMRTDInfo(const char *Cmd) {
             // TODO check MRZ checksums?
             if (!validate_date(dob, 6)) {
                 PrintAndLogEx(ERR, "Date of birth date format is incorrect, cannot continue.");
-                PrintAndLogEx(HINT, "Use the format YYMMDD.");
+                PrintAndLogEx(HINT, "Hint: Use the format YYMMDD.");
                 error = true;
             }
             if (!validate_date(expiry, 6)) {
                 PrintAndLogEx(ERR, "Expiry date format is incorrect, cannot continue.");
-                PrintAndLogEx(HINT, "Use the format YYMMDD.");
+                PrintAndLogEx(HINT, "Hint: Use the format YYMMDD.");
                 error = true;
             }
         }
@@ -2426,8 +2449,9 @@ static int CmdHFeMRTDList(const char *Cmd) {
 
 static command_t CommandTable[] = {
     {"help",    CmdHelp,           AlwaysAvailable, "This help"},
+    {"-----------", CmdHelp,           IfPm3Iso14443,   "------------------- " _CYAN_("Operations") " -------------------"},
     {"dump",    CmdHFeMRTDDump,    IfPm3Iso14443,   "Dump eMRTD files to binary files"},
-    {"info",    CmdHFeMRTDInfo,    AlwaysAvailable, "Display info about an eMRTD"},
+    {"info",        CmdHFeMRTDInfo,    AlwaysAvailable, "Tag information"},
     {"list",    CmdHFeMRTDList,    AlwaysAvailable, "List ISO 14443A/7816 history"},
     {NULL, NULL, NULL, NULL}
 };
