@@ -53,6 +53,8 @@ int ul_print_type(uint64_t tagtype, uint8_t spaces);
 void mfu_print_dump(mfu_dump_t *card, uint16_t pages, uint8_t startpage, bool dense_output);
 int ul_read_uid(uint8_t *uid);
 int trace_mfuc_try_default_3des_keys(uint8_t **correct_key, int state, uint8_t (*authdata)[16]);
+int mfu_get_version_uid(uint8_t *version, uint8_t *uid);
+int mfuc_test_authentication_support(void);
 
 int CmdHFMFUltra(const char *Cmd);
 int CmdHF14MfuNDEFRead(const char *Cmd);
@@ -96,6 +98,8 @@ int CmdHF14MfUTamper(const char *Cmd);
 #define MFU_TT_MAGIC_4          0x400000000ULL
 #define MFU_TT_MAGIC_4_GDM      0x800000000ULL
 #define MFU_TT_MAGIC_NTAG21X    0x1000000000ULL
+#define MFU_TT_ST25TN512        0x2000000000ULL
+#define MFU_TT_ST25TN01K        0x4000000000ULL
 #define MFU_TT_UL_MAGIC         (MFU_TT_UL | MFU_TT_MAGIC)
 #define MFU_TT_UL_C_MAGIC       (MFU_TT_UL_C | MFU_TT_MAGIC)
 // Don't forget to fill UL_TYPES_ARRAY and UL_MEMORY_ARRAY if new types are added
