@@ -22,6 +22,10 @@
 #include <string.h>
 #include <ctype.h>
 
+#ifdef HAVE_PYTHON
+#include <Python.h>
+#endif
+
 #include "cmdparser.h"      // command_t
 #include "cliparser.h"
 #include "comms.h"
@@ -41,6 +45,8 @@
 #include "proxgui.h"
 #include "graph.h"          // for graph data
 
+#include "lua.h"
+
 static int CmdHelp(const char *Cmd);
 
 static void lookup_chipid_short(uint32_t iChipID, uint32_t mem_used) {
@@ -145,8 +151,6 @@ static void lookup_chipid_short(uint32_t iChipID, uint32_t mem_used) {
                   , mem_avail
                   , mem_avail == 0 ? 0.0f : (float)mem_used / (mem_avail * 1024) * 100
                  );
-
-    PrintAndLogEx(NORMAL, "");
 }
 
 static void lookupChipID(uint32_t iChipID, uint32_t mem_used) {
@@ -683,8 +687,8 @@ static int CmdReadmem(const char *Cmd) {
     CLIParserFree(ctx);
 
     uint8_t *buffer = calloc(len, sizeof(uint8_t));
-    if (!buffer) {
-        PrintAndLogEx(ERR, "error, cannot allocate memory ");
+    if (buffer == NULL) {
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return PM3_EMALLOC;
     }
 
@@ -924,15 +928,19 @@ static int CmdTune(const char *Cmd) {
     SendCommandNG(CMD_MEASURE_ANTENNA_TUNING, NULL, 0);
     PacketResponseNG resp;
     PrintAndLogEx(INPLACE, "% 3i", timeout_max - timeout);
-    while (!WaitForResponseTimeout(CMD_MEASURE_ANTENNA_TUNING, &resp, 500)) {
+
+    while (WaitForResponseTimeout(CMD_MEASURE_ANTENNA_TUNING, &resp, 500) == false) {
+
         fflush(stdout);
         if (timeout >= timeout_max) {
             PrintAndLogEx(WARNING, "\nNo response from Proxmark3. Aborting...");
             return PM3_ETIMEOUT;
         }
+
         timeout++;
         PrintAndLogEx(INPLACE, "% 3i", timeout_max - timeout);
     }
+
     PrintAndLogEx(NORMAL, "");
 
     if (resp.status != PM3_SUCCESS) {
@@ -1352,7 +1360,7 @@ static int CmdConnect(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_str0("p", "port", NULL, "Serial port to connect to, else retry the last used one"),
+        arg_str0("p", "port", "<str>", "Serial port to connect to, else retry the last used one"),
         arg_u64_0("b", "baud", "<dec>", "Baudrate"),
         arg_param_end
     };
@@ -1440,7 +1448,7 @@ int set_fpga_mode(uint8_t mode) {
     SendCommandNG(CMD_SET_FPGAMODE, d, sizeof(d));
     PacketResponseNG resp;
     if (WaitForResponseTimeout(CMD_SET_FPGAMODE, &resp, 1000) == false) {
-        PrintAndLogEx(WARNING, "command execution timeout");
+        PrintAndLogEx(WARNING, "command execution time out");
         return PM3_ETIMEOUT;
     }
     if (resp.status != PM3_SUCCESS) {
@@ -1542,7 +1550,8 @@ int CmdHW(const char *Cmd) {
 #endif
 
 void pm3_version_short(void) {
-    PrintAndLogEx(NORMAL, "  [ " _CYAN_("Proxmark3 RFID instrument") " ]");
+//    PrintAndLogEx(NORMAL, "  [ " _CYAN_("Proxmark3 RFID instrument") " ]");
+    PrintAndLogEx(NORMAL, "  [ " _CYAN_("Proxmark3") " ]");
     PrintAndLogEx(NORMAL, "");
 
     if (g_session.pm3_present) {
@@ -1564,43 +1573,6 @@ void pm3_version_short(void) {
 
             lookup_chipid_short(payload->id, payload->section_size);
 
-            // client
-            char temp[PM3_CMD_DATA_SIZE - 12]; // same limit as for ARM image
-            format_version_information_short(temp, sizeof(temp), &g_version_information);
-            PrintAndLogEx(NORMAL, "    Client.... %s", temp);
-
-            bool armsrc_mismatch = false;
-            char *ptr = strstr(payload->versionstr, " os: ");
-            if (ptr != NULL) {
-                ptr = strstr(ptr, "\n");
-                if ((ptr != NULL) && (strlen(g_version_information.armsrc) == 9)) {
-                    if (strncmp(ptr - 9, g_version_information.armsrc, 9) != 0) {
-                        armsrc_mismatch = true;
-                    }
-                }
-            }
-
-            // bootrom
-            ptr = strstr(payload->versionstr, " bootrom: ");
-            if (ptr != NULL) {
-                char *ptr_end = strstr(ptr, "\n");
-                if (ptr_end != NULL) {
-                    uint8_t len = ptr_end - 19 - ptr;
-                    PrintAndLogEx(NORMAL, "    Bootrom... %.*s", len, ptr + 10);
-                }
-            }
-
-            // os:
-            ptr = strstr(payload->versionstr, " os: ");
-            if (ptr != NULL) {
-                char *ptr_end = strstr(ptr, "\n");
-                if (ptr_end != NULL) {
-                    uint8_t len = ptr_end - 14 - ptr;
-                    PrintAndLogEx(NORMAL, "    OS........ %.*s", len, ptr + 5);
-                }
-            }
-
-
             if (IfPm3Rdv4Fw()) {
 
                 bool is_genuine_rdv4 = false;
@@ -1616,7 +1588,43 @@ void pm3_version_short(void) {
             } else {
                 PrintAndLogEx(NORMAL, "    Target.... %s", _YELLOW_("PM3 GENERIC"));
             }
+            PrintAndLogEx(NORMAL, "");
 
+            // client
+            char temp[PM3_CMD_DATA_SIZE - 12]; // same limit as for ARM image
+            format_version_information_short(temp, sizeof(temp), &g_version_information);
+            PrintAndLogEx(NORMAL, "    Client.... %s", temp);
+
+            bool armsrc_mismatch = false;
+            char *ptr = strstr(payload->versionstr, "OS......... ");
+            if (ptr != NULL) {
+                ptr = strstr(ptr, "\n");
+                if ((ptr != NULL) && (strlen(g_version_information.armsrc) == 9)) {
+                    if (strncmp(ptr - 9, g_version_information.armsrc, 9) != 0) {
+                        armsrc_mismatch = true;
+                    }
+                }
+            }
+
+            // bootrom
+            ptr = strstr(payload->versionstr, "Bootrom.... ");
+            if (ptr != NULL) {
+                char *ptr_end = strstr(ptr, "\n");
+                if (ptr_end != NULL) {
+                    uint8_t len = ptr_end - 12 - ptr;
+                    PrintAndLogEx(NORMAL, "    Bootrom... %.*s", len, ptr + 12);
+                }
+            }
+
+            // os:
+            ptr = strstr(payload->versionstr, "OS......... ");
+            if (ptr != NULL) {
+                char *ptr_end = strstr(ptr, "\n");
+                if (ptr_end != NULL) {
+                    uint8_t len = ptr_end - 12 - ptr;
+                    PrintAndLogEx(NORMAL, "    OS........ %.*s", len, ptr + 12);
+                }
+            }
             PrintAndLogEx(NORMAL, "");
 
             if (armsrc_mismatch) {
@@ -1636,19 +1644,20 @@ void pm3_version(bool verbose, bool oneliner) {
     if (oneliner) {
         // For "proxmark3 -v", simple printf, avoid logging
         FormatVersionInformation(temp, sizeof(temp), "Client: ", &g_version_information);
-        PrintAndLogEx(NORMAL, "%s compiled with " PM3CLIENTCOMPILER __VERSION__ " OS:" PM3HOSTOS " ARCH:" PM3HOSTARCH "\n", temp);
+        PrintAndLogEx(NORMAL, "%s compiler: " PM3CLIENTCOMPILER __VERSION__ " OS:" PM3HOSTOS " ARCH:" PM3HOSTARCH "\n", temp);
         return;
     }
 
-    if (!verbose)
+    if (!verbose) {
         return;
+    }
 
-    PrintAndLogEx(NORMAL, "\n [ " _YELLOW_("Proxmark3 RFID instrument") " ]");
+    PrintAndLogEx(NORMAL, "\n [ " _CYAN_("Proxmark3") " ]");
     PrintAndLogEx(NORMAL, "\n [ " _YELLOW_("Client") " ]");
     FormatVersionInformation(temp, sizeof(temp), "  ", &g_version_information);
     PrintAndLogEx(NORMAL, "%s", temp);
-    PrintAndLogEx(NORMAL, "  compiled with............. " PM3CLIENTCOMPILER __VERSION__);
-    PrintAndLogEx(NORMAL, "  platform.................. " PM3HOSTOS " / " PM3HOSTARCH);
+    PrintAndLogEx(NORMAL, "  Compiler.................. " PM3CLIENTCOMPILER __VERSION__);
+    PrintAndLogEx(NORMAL, "  Platform.................. " PM3HOSTOS " / " PM3HOSTARCH);
 #if defined(HAVE_READLINE)
     PrintAndLogEx(NORMAL, "  Readline support.......... " _GREEN_("present"));
 #elif defined(HAVE_LINENOISE)
@@ -1662,28 +1671,29 @@ void pm3_version(bool verbose, bool oneliner) {
     PrintAndLogEx(NORMAL, "  QT GUI support............ " _YELLOW_("absent"));
 #endif
 #ifdef HAVE_BLUEZ
-    PrintAndLogEx(NORMAL, "  native BT support......... " _GREEN_("present"));
+    PrintAndLogEx(NORMAL, "  Native BT support......... " _GREEN_("present"));
 #else
-    PrintAndLogEx(NORMAL, "  native BT support......... " _YELLOW_("absent"));
+    PrintAndLogEx(NORMAL, "  Native BT support......... " _YELLOW_("absent"));
 #endif
 #ifdef HAVE_PYTHON
-    PrintAndLogEx(NORMAL, "  Python script support..... " _GREEN_("present"));
+    PrintAndLogEx(NORMAL, "  Python script support..... " _GREEN_("present") " ( " _YELLOW_(PY_VERSION) " )");
 #else
     PrintAndLogEx(NORMAL, "  Python script support..... " _YELLOW_("absent"));
 #endif
-#ifdef HAVE_LUA_SWIG
-    PrintAndLogEx(NORMAL, "  Lua SWIG support.......... " _GREEN_("present"));
-#else
-    PrintAndLogEx(NORMAL, "  Lua SWIG support.......... " _YELLOW_("absent"));
-#endif
 #ifdef HAVE_PYTHON_SWIG
     PrintAndLogEx(NORMAL, "  Python SWIG support....... " _GREEN_("present"));
 #else
     PrintAndLogEx(NORMAL, "  Python SWIG support....... " _YELLOW_("absent"));
 #endif
+    PrintAndLogEx(NORMAL, "  Lua script support........ " _GREEN_("present") " ( " _YELLOW_("%s.%s.%s") " )", LUA_VERSION_MAJOR, LUA_VERSION_MINOR, LUA_VERSION_RELEASE);
+#ifdef HAVE_LUA_SWIG
+    PrintAndLogEx(NORMAL, "  Lua SWIG support.......... " _GREEN_("present"));
+#else
+    PrintAndLogEx(NORMAL, "  Lua SWIG support.......... " _YELLOW_("absent"));
+#endif
 
     if (g_session.pm3_present) {
-        PrintAndLogEx(NORMAL, "\n [ " _YELLOW_("Proxmark3") " ]");
+        PrintAndLogEx(NORMAL, "\n [ " _YELLOW_("Model") " ]");
 
         PacketResponseNG resp;
         clearCommandBuffer();
@@ -1701,15 +1711,15 @@ void pm3_version(bool verbose, bool oneliner) {
                     }
                 }
 
-                PrintAndLogEx(NORMAL, "  device.................... %s", (is_genuine_rdv4) ? _GREEN_("RDV4") : _RED_("device / fw mismatch"));
-                PrintAndLogEx(NORMAL, "  firmware.................. %s", (is_genuine_rdv4) ? _GREEN_("RDV4") : _YELLOW_("RDV4"));
-                PrintAndLogEx(NORMAL, "  external flash............ %s", IfPm3Flash() ? _GREEN_("present") : _YELLOW_("absent"));
-                PrintAndLogEx(NORMAL, "  smartcard reader.......... %s", IfPm3Smartcard() ? _GREEN_("present") : _YELLOW_("absent"));
+                PrintAndLogEx(NORMAL, "  Device.................... %s", (is_genuine_rdv4) ? _GREEN_("RDV4") : _RED_("device / fw mismatch"));
+                PrintAndLogEx(NORMAL, "  Firmware.................. %s", (is_genuine_rdv4) ? _GREEN_("RDV4") : _YELLOW_("RDV4"));
+                PrintAndLogEx(NORMAL, "  External flash............ %s", IfPm3Flash() ? _GREEN_("present") : _YELLOW_("absent"));
+                PrintAndLogEx(NORMAL, "  Smartcard reader.......... %s", IfPm3Smartcard() ? _GREEN_("present") : _YELLOW_("absent"));
                 PrintAndLogEx(NORMAL, "  FPC USART for BT add-on... %s", IfPm3FpcUsartHost() ? _GREEN_("present") : _YELLOW_("absent"));
             } else {
-                PrintAndLogEx(NORMAL, "  firmware.................. %s", _YELLOW_("PM3 GENERIC"));
+                PrintAndLogEx(NORMAL, "  Firmware.................. %s", _YELLOW_("PM3 GENERIC"));
                 if (IfPm3Flash()) {
-                    PrintAndLogEx(NORMAL, "  external flash............ %s", _GREEN_("present"));
+                    PrintAndLogEx(NORMAL, "  External flash............ %s", _GREEN_("present"));
                 }
 
                 if (IfPm3FpcUsartHost()) {
@@ -1733,7 +1743,7 @@ void pm3_version(bool verbose, bool oneliner) {
             struct p *payload = (struct p *)&resp.data.asBytes;
 
             bool armsrc_mismatch = false;
-            char *ptr = strstr(payload->versionstr, " os: ");
+            char *ptr = strstr(payload->versionstr, "OS......... ");
             if (ptr != NULL) {
                 ptr = strstr(ptr, "\n");
                 if ((ptr != NULL) && (strlen(g_version_information.armsrc) == 9)) {
