@@ -40,6 +40,8 @@
 #include "cmdlfem4x70.h"    // for em4x70
 #include "cmdlfhid.h"       // for hid menu
 #include "cmdlfhitag.h"     // for hitag menu
+#include "cmdlfhitaghts.h"  // for hitag S sub commands
+#include "cmdlfhitagu.h"    // for hitag µ sub commands
 #include "cmdlfidteck.h"    // for idteck menu
 #include "cmdlfio.h"        // for ioprox menu
 #include "cmdlfcotag.h"     // for COTAG menu
@@ -176,7 +178,7 @@ static int CmdLFTune(const char *Cmd) {
 
     SendCommandNG(CMD_MEASURE_ANTENNA_TUNING_LF, params, sizeof(params));
     if (WaitForResponseTimeout(CMD_MEASURE_ANTENNA_TUNING_LF, &resp, 1000) == false) {
-        PrintAndLogEx(WARNING, "Timeout while waiting for Proxmark LF initialization, aborting");
+        PrintAndLogEx(WARNING, "timeout while waiting for Proxmark LF initialization, aborting");
         return PM3_ETIMEOUT;
     }
 
@@ -200,7 +202,7 @@ static int CmdLFTune(const char *Cmd) {
         SendCommandNG(CMD_MEASURE_ANTENNA_TUNING_LF, params, sizeof(params));
         if (WaitForResponseTimeout(CMD_MEASURE_ANTENNA_TUNING_LF, &resp, 1000) == false) {
             PrintAndLogEx(NORMAL, "");
-            PrintAndLogEx(WARNING, "Timeout while waiting for Proxmark LF measure, aborting");
+            PrintAndLogEx(WARNING, "timeout while waiting for Proxmark LF measure, aborting");
             break;
         }
 
@@ -227,7 +229,7 @@ static int CmdLFTune(const char *Cmd) {
     params[0] = 3;
     SendCommandNG(CMD_MEASURE_ANTENNA_TUNING_LF, params, sizeof(params));
     if (WaitForResponseTimeout(CMD_MEASURE_ANTENNA_TUNING_LF, &resp, 1000) == false) {
-        PrintAndLogEx(WARNING, "Timeout while waiting for Proxmark LF shutdown, aborting");
+        PrintAndLogEx(WARNING, "timeout while waiting for Proxmark LF shutdown, aborting");
         return PM3_ETIMEOUT;
     }
 
@@ -241,6 +243,8 @@ static int CmdLFTune(const char *Cmd) {
     return PM3_SUCCESS;
 }
 
+#define PAYLOAD_HEADER_SIZE (12 + (3 * LF_CMDREAD_MAX_EXTRA_SYMBOLS))
+
 /* send a LF command before reading */
 int CmdLFCommandRead(const char *Cmd) {
     CLIParserContext *ctx;
@@ -274,7 +278,7 @@ int CmdLFCommandRead(const char *Cmd) {
     CLIExecWithReturn(ctx, Cmd, argtable, false);
     uint32_t delay = arg_get_u32_def(ctx, 1, 0);
 
-    char cmd[128] = {0};
+    char cmd[PM3_CMD_DATA_SIZE - PAYLOAD_HEADER_SIZE] = {0};
     int cmd_len = sizeof(cmd) - 1; // CLIGetStrWithReturn does not guarantee string to be null-terminated
     CLIGetStrWithReturn(ctx, 2, (uint8_t *)cmd, &cmd_len);
 
@@ -285,17 +289,18 @@ int CmdLFCommandRead(const char *Cmd) {
     uint16_t period_1 = arg_get_u32_def(ctx, 4, 0);
     uint16_t period_0 = arg_get_u32_def(ctx, 5, 0);
     uint32_t samples = arg_get_u32_def(ctx, 6, 0);
+
     bool verbose = arg_get_lit(ctx, 7);
     bool keep_field_on = arg_get_lit(ctx, 8);
     bool add_crc_ht = arg_get_lit(ctx, 9);
     bool cm = arg_get_lit(ctx, 10);
+
     CLIParserFree(ctx);
 
     if (g_session.pm3_present == false) {
         return PM3_ENOTTY;
     }
 
-#define PAYLOAD_HEADER_SIZE (12 + (3 * LF_CMDREAD_MAX_EXTRA_SYMBOLS))
     struct p {
         uint32_t delay;
         uint16_t period_0;
@@ -316,15 +321,23 @@ int CmdLFCommandRead(const char *Cmd) {
     memset(payload.symbol_extra, 0, sizeof(payload.symbol_extra));
     memset(payload.period_extra, 0, sizeof(payload.period_extra));
 
-    if (add_crc_ht && (cmd_len <= 120)) {
+    if (cmd_len > sizeof(payload.data) - 8 * add_crc_ht - 1) {
+        PrintAndLogEx(ERR, "cmd too long, max length is %zu", sizeof(cmd) - 1);
+        return PM3_EINVARG;
+    }
+
+    if (add_crc_ht) {
+
         // Hitag 1, Hitag S, ZX8211
         // width=8 poly=0x1d init=0xff refin=false refout=false xorout=0x00 check=0xb4 residue=0x00 name="CRC-8/HITAG"
         crc_t crc;
+        crc_init_ref(&crc, 8, 0x1d, 0xff, 0, false, false);
+
         uint8_t data = 0;
         uint8_t n = 0;
-        crc_init_ref(&crc, 8, 0x1d, 0xff, 0, false, false);
-        uint8_t i;
-        for (i = 0; i < cmd_len; i++) {
+
+        for (int i = 0; i < cmd_len; i++) {
+
             if ((cmd[i] != '0') && (cmd[i] != '1')) {
                 // avoid include 'W0S' in crc
                 crc_init_ref(&crc, 8, 0x1d, 0xff, 0, false, false);
@@ -332,18 +345,22 @@ int CmdLFCommandRead(const char *Cmd) {
                 data = 0;
                 continue;
             }
+
             data <<= 1;
             data += cmd[i] - '0';
-            n += 1;
+            n++;
+
             if (n == 8) {
                 crc_update2(&crc, data, n);
                 n = 0;
                 data = 0;
             }
         }
+
         if (n > 0) {
             crc_update2(&crc, data, n);
         }
+
         uint8_t crc_final = crc_finish(&crc);
         for (int j = 7; j >= 0; j--) {
             cmd[cmd_len] = ((crc_final >> j) & 1) ? '1' : '0';
@@ -412,30 +429,23 @@ int CmdLFCommandRead(const char *Cmd) {
         clearCommandBuffer();
         SendCommandNG(CMD_LF_MOD_THEN_ACQ_RAW_ADC, (uint8_t *)&payload, PAYLOAD_HEADER_SIZE + cmd_len + 1);
 
-        PacketResponseNG resp;
         // init to ZERO
-        resp.cmd = 0,
-        resp.length = 0,
-        resp.magic = 0,
-        resp.status = 0,
-        resp.crc = 0,
-        resp.ng = false,
-        resp.oldarg[0] = 0;
-        resp.oldarg[1] = 0;
-        resp.oldarg[2] = 0;
-        memset(resp.data.asBytes, 0, PM3_CMD_DATA_SIZE);
+        PacketResponseNG resp;
+        memset(&resp, 0, sizeof(resp));
 
         i = 10;
         // 20sec wait loop
-        while (!WaitForResponseTimeout(CMD_LF_MOD_THEN_ACQ_RAW_ADC, &resp, 2000) && i != 0) {
+        while (WaitForResponseTimeout(CMD_LF_MOD_THEN_ACQ_RAW_ADC, &resp, 2000) == false && i != 0) {
             if (verbose) {
                 PrintAndLogEx(NORMAL, "." NOLF);
             }
             i--;
         }
+
         if (verbose) {
             PrintAndLogEx(NORMAL, "");
         }
+
         if (resp.status != PM3_SUCCESS) {
             PrintAndLogEx(WARNING, "command failed.");
             return PM3_ESOFT;
@@ -448,7 +458,7 @@ int CmdLFCommandRead(const char *Cmd) {
             getSamples(samples, false);
             ret = PM3_SUCCESS;
         } else {
-            PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+            PrintAndLogEx(WARNING, "timeout while waiting for reply");
             return PM3_ETIMEOUT;
         }
 
@@ -466,7 +476,7 @@ int CmdFlexdemod(const char *Cmd) {
 
     int *data = calloc(g_GraphTraceLen, sizeof(int));
     if (data == NULL) {
-        PrintAndLogEx(FAILED, "failed to allocate memory");
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return PM3_EMALLOC;
     }
     memcpy(data, g_GraphBuffer, g_GraphTraceLen);
@@ -587,7 +597,7 @@ int lf_getconfig(sample_config *config) {
 
     SendCommandNG(CMD_LF_SAMPLING_GET_CONFIG, NULL, 0);
     PacketResponseNG resp;
-    if (!WaitForResponseTimeout(CMD_LF_SAMPLING_GET_CONFIG, &resp, 2000)) {
+    if (WaitForResponseTimeout(CMD_LF_SAMPLING_GET_CONFIG, &resp, 2000) == false) {
         PrintAndLogEx(WARNING, "command execution time out");
         return PM3_ETIMEOUT;
     }
@@ -752,7 +762,7 @@ static int lf_read_internal(bool realtime, bool verbose, uint64_t samples) {
     if (realtime) {
         uint8_t *realtimeBuf = calloc(samples, sizeof(uint8_t));
         if (realtimeBuf == NULL) {
-            PrintAndLogEx(FAILED, "failed to allocate memory");
+            PrintAndLogEx(WARNING, "Failed to allocate memory");
             return PM3_EMALLOC;
         }
 
@@ -789,10 +799,12 @@ static int lf_read_internal(bool realtime, bool verbose, uint64_t samples) {
         payload.samples = (samples > MAX_LF_SAMPLES) ? MAX_LF_SAMPLES : samples;
         SendCommandNG(CMD_LF_ACQ_RAW_ADC, (uint8_t *)&payload, sizeof(payload));
         PacketResponseNG resp;
+
         if (is_trigger_threshold_set) {
             WaitForResponse(CMD_LF_ACQ_RAW_ADC, &resp);
         } else {
-            if (!WaitForResponseTimeout(CMD_LF_ACQ_RAW_ADC, &resp, 2500)) {
+
+            if (WaitForResponseTimeout(CMD_LF_ACQ_RAW_ADC, &resp, 2500) == false) {
                 PrintAndLogEx(WARNING, "(lf_read) command execution time out");
                 return PM3_ETIMEOUT;
             }
@@ -879,7 +891,7 @@ int lf_sniff(bool realtime, bool verbose, uint64_t samples) {
     if (realtime) {
         uint8_t *realtimeBuf = calloc(samples, sizeof(uint8_t));
         if (realtimeBuf == NULL) {
-            PrintAndLogEx(FAILED, "failed to allocate memory");
+            PrintAndLogEx(WARNING, "Failed to allocate memory");
             return PM3_EMALLOC;
         }
 
@@ -1119,9 +1131,10 @@ int CmdLFfskSim(const char *Cmd) {
     uint8_t fchigh = arg_get_u32_def(ctx, 3, 0);
     bool separator = arg_get_lit(ctx, 4);
 
-    int raw_len = 64;
-    char raw[64] = {0};
+    char raw[65] = {0};
+    int raw_len = sizeof(raw) - 1; // CLIGetStrWithReturn does not guarantee string to be null-terminated
     CLIGetStrWithReturn(ctx, 5, (uint8_t *)raw, &raw_len);
+
     bool verbose = arg_get_lit(ctx, 6);
     CLIParserFree(ctx);
 
@@ -1229,9 +1242,10 @@ int CmdLFaskSim(const char *Cmd) {
     bool use_ar = arg_get_lit(ctx, 5);
     bool separator = arg_get_lit(ctx, 6);
 
-    int raw_len = 64;
-    char raw[64] = {0};
+    char raw[65] = {0};
+    int raw_len = sizeof(raw) - 1; // CLIGetStrWithReturn does not guarantee string to be null-terminated
     CLIGetStrWithReturn(ctx, 7, (uint8_t *)raw, &raw_len);
+
     bool verbose = arg_get_lit(ctx, 8);
     CLIParserFree(ctx);
 
@@ -1331,17 +1345,22 @@ int CmdLFpskSim(const char *Cmd) {
         arg_lit0("v", "verbose", "verbose output"),
         arg_param_end
     };
+
     CLIExecWithReturn(ctx, Cmd, argtable, true);
+
     bool use_psk1 = arg_get_lit(ctx, 1);
     bool use_psk2 = arg_get_lit(ctx, 2);
     bool use_psk3 = arg_get_lit(ctx, 3);
     bool invert = arg_get_lit(ctx, 4);
+
     uint8_t clk = arg_get_u32_def(ctx, 5, 0);
     uint8_t carrier = arg_get_u32_def(ctx, 6, 2);
-    int raw_len = 64;
-    char raw[64] = {0};
+
+    char raw[65] = {0};
+    int raw_len = sizeof(raw) - 1; // CLIGetStrWithReturn does not guarantee string to be null-terminated
     CLIGetStrWithReturn(ctx, 7, (uint8_t *)raw, &raw_len);
     bool verbose = arg_get_lit(ctx, 8);
+
     CLIParserFree(ctx);
 
     if ((use_psk1 + use_psk2 + use_psk3) > 1) {
@@ -1542,7 +1561,9 @@ static bool check_chiptype(bool getDeviceData) {
 
     bool retval = false;
 
-    if (!getDeviceData) return retval;
+    if (getDeviceData == false) {
+        return retval;
+    }
 
     //Save the state of the Graph and Demod Buffers
     buffer_savestate_t saveState_gb = save_bufferS32(g_GraphBuffer, g_GraphTraceLen);
@@ -1555,7 +1576,7 @@ static bool check_chiptype(bool getDeviceData) {
     uint32_t word = 0;
     if (IfPm3EM4x50() && em4x05_isblock0(&word)) {
         PrintAndLogEx(SUCCESS, "Chipset detection: " _GREEN_("EM4x05 / EM4x69"));
-        PrintAndLogEx(HINT, "Hint: try " _YELLOW_("`lf em 4x05`") " commands");
+        PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("lf em 4x05") "` commands");
         retval = true;
         goto out;
     }
@@ -1563,16 +1584,45 @@ static bool check_chiptype(bool getDeviceData) {
     //check for t55xx chip...
     if (tryDetectP1(true)) {
         PrintAndLogEx(SUCCESS, "Chipset detection: " _GREEN_("T55xx"));
-        PrintAndLogEx(HINT, "Hint: try " _YELLOW_("`lf t55xx`") " commands");
+        PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("lf t55xx") "` commands");
         retval = true;
         goto out;
     }
 
+
+    if (IfPm3Hitag()) {
+
+        // Hitag 2
+        if (ht2_read_uid() == PM3_SUCCESS) {
+            PrintAndLogEx(SUCCESS, "Chipset detection: " _GREEN_("Hitag 2"));
+            PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("lf hitag") "` commands");
+            retval = true;
+            goto out;
+        }
+
+        // Hitag S
+        if (read_hts_uid() == PM3_SUCCESS) {
+            PrintAndLogEx(SUCCESS, "Chipset detection: " _GREEN_("Hitag 1/S / 82xx"));
+            PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("lf hitag hts") "` commands");
+            retval = true;
+            goto out;
+        }
+
+        // Hitag µ
+        if (read_htu_uid() == PM3_SUCCESS) {
+            PrintAndLogEx(SUCCESS, "Chipset detection: " _GREEN_("Hitag µ / 8265"));
+            PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("lf hitag htu") "` commands");
+            retval = true;
+            goto out;
+        }
+    }
+
+
 #if !defined ICOPYX
     // check for em4x50 chips
     if (IfPm3EM4x50() && detect_4x50_block()) {
         PrintAndLogEx(SUCCESS, "Chipset detection: " _GREEN_("EM4x50"));
-        PrintAndLogEx(HINT, "Hint: try " _YELLOW_("`lf em 4x50`") " commands");
+        PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("lf em 4x50") "` commands");
         retval = true;
         goto out;
     }
@@ -1580,7 +1630,7 @@ static bool check_chiptype(bool getDeviceData) {
     // check for em4x70 chips
     if (IfPm3EM4x70() && detect_4x70_block()) {
         PrintAndLogEx(SUCCESS, "Chipset detection: " _GREEN_("EM4x70"));
-        PrintAndLogEx(HINT, "Hint: try " _YELLOW_("`lf em 4x70`") " commands");
+        PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("lf em 4x70") "` commands");
         retval = true;
         goto out;
     }
@@ -1662,8 +1712,9 @@ int CmdLFfind(const char *Cmd) {
     CLIParserFree(ctx);
     int found = 0;
     bool is_online = (g_session.pm3_present && (use_gb == false));
-    if (is_online)
+    if (is_online) {
         lf_read(false, 30000);
+    }
 
     size_t min_length = 2000;
     if (g_GraphTraceLen < min_length) {
@@ -1681,16 +1732,18 @@ int CmdLFfind(const char *Cmd) {
     PrintAndLogEx(INFO, _CYAN_("Checking for known tags..."));
     PrintAndLogEx(INFO, "");
 
+    int retval = PM3_SUCCESS;
+
     // only run these tests if device is online
     if (is_online) {
 
         if (IfPm3Hitag()) {
-            if (readHitagUid() == PM3_SUCCESS) {
-                PrintAndLogEx(SUCCESS, "\nValid " _GREEN_("Hitag") " found!");
+            if (ht2_read_paxton() == PM3_SUCCESS) {
+                PrintAndLogEx(SUCCESS, "\nValid " _GREEN_("Paxton ID") " found!");
                 if (search_cont) {
                     found++;
                 } else {
-                    return PM3_SUCCESS;
+                    goto out;
                 }
             }
         }
@@ -1734,7 +1787,7 @@ int CmdLFfind(const char *Cmd) {
 
             PrintAndLogEx(NORMAL, "");
             PrintAndLogEx(FAILED, _RED_("No data found!"));
-            PrintAndLogEx(HINT, "Maybe not an LF tag?");
+            PrintAndLogEx(HINT, "Hint: Maybe not an LF tag?");
             PrintAndLogEx(NORMAL, "");
             if (search_cont == 0) {
                 return PM3_ESOFT;
@@ -1742,8 +1795,6 @@ int CmdLFfind(const char *Cmd) {
         }
     }
 
-    int retval = PM3_SUCCESS;
-
     // ask / man
     if (demodEM410x(true) == PM3_SUCCESS) {
         PrintAndLogEx(SUCCESS, "\nValid " _GREEN_("EM410x ID") " found!");
