@@ -16,6 +16,8 @@
 // Low frequency Hitag support
 //-----------------------------------------------------------------------------
 #include "cmdlfhitag.h"
+#include "cmdlfhitaghts.h"
+#include "cmdlfhitagu.h"
 #include <ctype.h>
 #include "cmdparser.h"  // command_t
 #include "comms.h"
@@ -31,10 +33,13 @@
 #include "pm3_cmd.h"    // return codes
 #include "hitag2/hitag2_crypto.h"
 #include "util_posix.h"             // msclock
-#include "cmdlfhitaghts.h"
 
 static int CmdHelp(const char *Cmd);
 
+static const uint8_t ht2_default_keys[] = {
+    0xBD, 0xF5, 0xE8, 0x46      // PAXTON
+};
+
 static const char *getHitagTypeStr(uint32_t uid) {
     //uid s/n        ********
     uint8_t type = (uid >> 4) & 0xF;
@@ -75,11 +80,11 @@ static size_t nbytes(size_t nbits) {
 */
 
 static int CmdLFHitagList(const char *Cmd) {
-    return CmdTraceListAlias(Cmd, "lf hitag", "hitag2");
+    return CmdTraceListAlias(Cmd, "lf hitag", "ht2");
     /*
     uint8_t *got = calloc(PM3_CMD_DATA_SIZE, sizeof(uint8_t));
-    if (!got) {
-        PrintAndLogEx(WARNING, "Cannot allocate memory for trace");
+    if (got == NULL) {
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return PM3_EMALLOC;
     }
 
@@ -95,7 +100,7 @@ static int CmdLFHitagList(const char *Cmd) {
     if (traceLen > PM3_CMD_DATA_SIZE) {
         uint8_t *p = realloc(got, traceLen);
         if (p == NULL) {
-            PrintAndLogEx(WARNING, "Cannot allocate memory for trace");
+            PrintAndLogEx(WARNING, "Failed to allocate memory");
             free(got);
             return PM3_EMALLOC;
         }
@@ -205,7 +210,7 @@ static int CmdLFHitagList(const char *Cmd) {
     */
 }
 
-static void print_hitag2_paxton(const uint8_t *data) {
+static void print_hitag2_paxton(bool show_header, const uint8_t *data) {
 
     // if the pwd isn't..
     if (memcmp(data + 4, "\xBD\xF5\xE8\x46", 4)) {
@@ -263,10 +268,14 @@ static void print_hitag2_paxton(const uint8_t *data) {
         }
     }
 
-    PrintAndLogEx(INFO, "");
-    PrintAndLogEx(INFO, "--- " _CYAN_("Possible de-scramble patterns") " -------------");
+    if (show_header) {
+        PrintAndLogEx(INFO, "");
+        PrintAndLogEx(INFO, "--- " _CYAN_("Possible de-scramble patterns") " -------------");
+    }
     PrintAndLogEx(SUCCESS, "Paxton id... %" PRIu64 " | 0x%" PRIx64 "  ( %s )", paxton_id, paxton_id, formfactor);
-    PrintAndLogEx(INFO, "");
+    if (show_header) {
+        PrintAndLogEx(INFO, "");
+    }
 }
 
 static void print_hitag2_configuration(uint32_t uid, uint8_t config) {
@@ -480,13 +489,28 @@ static int ht2_check_dictionary(uint32_t key_count, uint8_t *keys,  uint8_t keyl
 
     uint8_t *pkeys = keys;
 
+    uint32_t toti = key_count;
+    uint32_t cnt = 0;
+
     while (key_count--) {
 
+        cnt++;
+
+        if (kbd_enter_pressed()) {
+            SendCommandNG(CMD_BREAK_LOOP, NULL, 0);
+            PrintAndLogEx(INFO, "User aborted");
+            break;
+        }
+
+        PrintAndLogEx(INPLACE, "Checking Keys %u / %u", cnt, toti);
+
+        msleep(30);
+
         if (keylen == 4) {
-            packet.cmd = RHT2F_PASSWORD;
+            packet.cmd = HT2F_PASSWORD;
             memcpy(packet.pwd, pkeys, keylen);
         } else {
-            packet.cmd = RHT2F_CRYPTO;
+            packet.cmd = HT2F_CRYPTO;
             memcpy(packet.key, pkeys, keylen);
         }
 
@@ -495,8 +519,8 @@ static int ht2_check_dictionary(uint32_t key_count, uint8_t *keys,  uint8_t keyl
         clearCommandBuffer();
         SendCommandNG(CMD_LF_HITAG_READER, (uint8_t *)&packet, sizeof(packet));
         PacketResponseNG resp;
-        if (WaitForResponseTimeout(CMD_LF_HITAG_READER, &resp, 2000) == false) {
-            PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+        if (WaitForResponseTimeout(CMD_LF_HITAG_READER, &resp, 4000) == false) {
+            PrintAndLogEx(WARNING, "timeout while waiting for reply");
             SendCommandNG(CMD_BREAK_LOOP, NULL, 0);
             return PM3_ETIMEOUT;
         }
@@ -565,6 +589,7 @@ void hitag2_annotate_plain(char *exp, size_t size, const uint8_t *cmd, uint8_t c
 
     char *binstr = (char *)calloc((cmdsize * 8) + 1, sizeof(uint8_t));
     if (binstr == NULL) {
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return;
     }
 
@@ -632,6 +657,7 @@ void annotateHitag2(char *exp, size_t size, const uint8_t *cmd, uint8_t cmdsize,
 
     char *binstr = (char *)calloc((cmdsize * 8) + 1, sizeof(uint8_t));
     if (binstr == NULL) {
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return;
     }
 
@@ -770,9 +796,6 @@ void annotateHitag2(char *exp, size_t size, const uint8_t *cmd, uint8_t cmdsize,
     free(binstr);
 }
 
-void annotateHitagS(char *exp, size_t size, const uint8_t *cmd, uint8_t cmdsize, bool is_response) {
-}
-
 static const char *identify_transponder_hitag2(uint32_t uid) {
 
     switch (uid) {
@@ -794,22 +817,22 @@ static const char *identify_transponder_hitag2(uint32_t uid) {
     return "";
 }
 
-static bool getHitag2Uid(uint32_t *uid) {
+static bool ht2_get_uid(uint32_t *uid) {
 
     lf_hitag_data_t packet;
     memset(&packet, 0, sizeof(packet));
-    packet.cmd = RHT2F_UID_ONLY;
+    packet.cmd = HT2F_UID_ONLY;
 
     clearCommandBuffer();
     SendCommandNG(CMD_LF_HITAG_READER, (uint8_t *) &packet, sizeof(packet));
     PacketResponseNG resp;
     if (WaitForResponseTimeout(CMD_LF_HITAG_READER, &resp, 1500) == false) {
-        PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+        PrintAndLogEx(WARNING, "timeout while waiting for reply");
         return false;
     }
 
     if (resp.status != PM3_SUCCESS) {
-        PrintAndLogEx(DEBUG, "DEBUG: Error - failed getting UID");
+        PrintAndLogEx(DEBUG, "DEBUG: Error - failed getting Hitag 2 UID");
         return false;
     }
 
@@ -835,7 +858,7 @@ static int CmdLFHitagInfo(const char *Cmd) {
 
     // read UID
     uint32_t uid = 0;
-    if (getHitag2Uid(&uid) == false) {
+    if (ht2_get_uid(&uid) == false) {
         return PM3_ESOFT;
     }
     // how to determine Hitag types?
@@ -886,7 +909,7 @@ static int CmdLFHitagReader(const char *Cmd) {
     do {
         // read UID
         uint32_t uid = 0;
-        if (getHitag2Uid(&uid)) {
+        if (ht2_get_uid(&uid)) {
             PrintAndLogEx(SUCCESS, "UID.... " _GREEN_("%08X"), uid);
         }
     } while (cm && kbd_enter_pressed() == false);
@@ -898,17 +921,13 @@ static int CmdLFHitagRd(const char *Cmd) {
 
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "lf hitag read",
-                  "Read Hitag memory. It support Hitag S and Hitag 2\n\n"
+                  "Read Hitag memory. It support Hitag 2\n\n"
                   "  Password mode:\n"
                   "    - default key 4D494B52 (MIKR)\n\n"
                   "  Crypto mode: \n"
                   "    - key format ISK high + ISK low\n"
                   "    - default key 4F4E4D494B52 (ONMIKR)\n"
                   ,
-                  "  lf hitag read --hts                         -> Hitag S, plain mode\n"
-                  "  lf hitag read --hts --nrar 0102030411223344 -> Hitag S, challenge mode\n"
-                  "  lf hitag read --hts --crypto                -> Hitag S, crypto mode, def key\n"
-                  "  lf hitag read --hts -k 4F4E4D494B52         -> Hitag S, crypto mode\n\n"
                   "  lf hitag read --ht2 --pwd                   -> Hitag 2, pwd mode, def key\n"
                   "  lf hitag read --ht2 -k 4D494B52             -> Hitag 2, pwd mode\n"
                   "  lf hitag read --ht2 --nrar 0102030411223344 -> Hitag 2, challenge mode\n"
@@ -918,7 +937,6 @@ static int CmdLFHitagRd(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("s", "hts", "Hitag S"),
         arg_lit0("2", "ht2", "Hitag 2"),
         arg_lit0(NULL, "pwd", "password mode"),
         arg_str0(NULL, "nrar", "<hex>", "nonce / answer writer, 8 hex bytes"),
@@ -931,25 +949,24 @@ static int CmdLFHitagRd(const char *Cmd) {
     CLIExecWithReturn(ctx, Cmd, argtable, false);
 
     bool use_ht1 = false; // not yet implemented
-    bool use_hts = arg_get_lit(ctx, 1);
-    bool use_ht2 = arg_get_lit(ctx, 2);
+    bool use_ht2 = arg_get_lit(ctx, 1);
     bool use_htm = false; // not yet implemented
 
     bool use_plain = false;
-    bool use_pwd = arg_get_lit(ctx, 3);
+    bool use_pwd = arg_get_lit(ctx, 2);
     uint8_t nrar[8];
     int nalen = 0;
-    int res = CLIParamHexToBuf(arg_get_str(ctx, 4), nrar, sizeof(nrar), &nalen);
+    int res = CLIParamHexToBuf(arg_get_str(ctx, 3), nrar, sizeof(nrar), &nalen);
     if (res != 0) {
         CLIParserFree(ctx);
         return PM3_EINVARG;
     }
     bool use_nrar = nalen > 0;
-    bool use_crypto = arg_get_lit(ctx, 5);
+    bool use_crypto = arg_get_lit(ctx, 4);
 
     uint8_t key[6];
     int keylen = 0;
-    res = CLIParamHexToBuf(arg_get_str(ctx, 6), key, sizeof(key), &keylen);
+    res = CLIParamHexToBuf(arg_get_str(ctx, 5), key, sizeof(key), &keylen);
     if (res != 0) {
         CLIParserFree(ctx);
         return PM3_EINVARG;
@@ -959,11 +976,11 @@ static int CmdLFHitagRd(const char *Cmd) {
     CLIParserFree(ctx);
 
     // sanity checks
-    if ((use_ht1 + use_ht2 + use_hts + use_htm) > 1) {
+    if ((use_ht1 + use_ht2 + use_htm) > 1) {
         PrintAndLogEx(ERR, "error, specify only one Hitag type");
         return PM3_EINVARG;
     }
-    if ((use_ht1 + use_ht2 + use_hts + use_htm) == 0) {
+    if ((use_ht1 + use_ht2 + use_htm) == 0) {
         PrintAndLogEx(ERR, "error, specify one Hitag type");
         return PM3_EINVARG;
     }
@@ -1000,16 +1017,7 @@ static int CmdLFHitagRd(const char *Cmd) {
         PrintAndLogEx(WARNING, "Specify only one authentication mode");
         return PM3_EINVARG;
     } else if (foo == 0) {
-        if (use_hts) {
-            use_plain = true;
-        } else {
-            PrintAndLogEx(WARNING, "Specify one authentication mode");
-            return PM3_EINVARG;
-        }
-    }
-
-    if (use_hts && use_pwd) { // not sure for the other types...
-        PrintAndLogEx(WARNING, "Chosen Hitag type does not have Password mode");
+        PrintAndLogEx(WARNING, "Specify one authentication mode");
         return PM3_EINVARG;
     }
 
@@ -1022,32 +1030,19 @@ static int CmdLFHitagRd(const char *Cmd) {
     memset(&packet, 0, sizeof(packet));
 
     int pm3cmd;
-    if (use_hts) {
-        // plain mode?
-        pm3cmd = CMD_LF_HITAGS_READ;
-    } else if (use_hts && use_nrar) {
-        pm3cmd = CMD_LF_HITAGS_READ;
-        packet.cmd = RHTSF_CHALLENGE;
-        memcpy(packet.NrAr, nrar, sizeof(packet.NrAr));
-
-    } else if (use_hts && use_crypto) {
-        pm3cmd = CMD_LF_HITAGS_READ;
-        packet.cmd = RHTSF_KEY;
-        memcpy(packet.key, key, sizeof(packet.key));
-
-    } else if (use_ht2 && use_pwd) {
+    if (use_ht2 && use_pwd) {
         pm3cmd = CMD_LF_HITAG_READER;
-        packet.cmd = RHT2F_PASSWORD;
+        packet.cmd = HT2F_PASSWORD;
         memcpy(packet.pwd, key, sizeof(packet.pwd));
 
     } else if (use_ht2 && use_nrar) {
         pm3cmd = CMD_LF_HITAG_READER;
-        packet.cmd = RHT2F_AUTHENTICATE;
+        packet.cmd = HT2F_AUTHENTICATE;
         memcpy(packet.NrAr, nrar, sizeof(packet.NrAr));
     } else if (use_ht2 && use_crypto) {
 
         pm3cmd = CMD_LF_HITAG_READER;
-        packet.cmd = RHT2F_CRYPTO;
+        packet.cmd = HT2F_CRYPTO;
         memcpy(packet.key, key, sizeof(packet.key));
     } else {
         PrintAndLogEx(WARNING, "Sorry, not yet implemented");
@@ -1059,7 +1054,7 @@ static int CmdLFHitagRd(const char *Cmd) {
 
     PacketResponseNG resp;
     if (WaitForResponseTimeout(pm3cmd, &resp, 2000) == false) {
-        PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+        PrintAndLogEx(WARNING, "timeout while waiting for reply");
         SendCommandNG(CMD_BREAK_LOOP, NULL, 0);
         return PM3_ETIMEOUT;
     }
@@ -1079,7 +1074,7 @@ static int CmdLFHitagRd(const char *Cmd) {
 
     if (use_ht2) {
         print_hitag2_blocks(data, HITAG2_MAX_BYTE_SIZE);
-        print_hitag2_paxton(data);
+        print_hitag2_paxton(true, data);
     } else {
         print_hex_break(data, HITAG_MAX_BYTE_SIZE, HITAG_BLOCK_SIZE);
     }
@@ -1142,13 +1137,13 @@ static int CmdLFHitag2CheckChallenges(const char *Cmd) {
 
     lf_hitag_data_t packet;
     memset(&packet, 0, sizeof(lf_hitag_data_t));
-    packet.cmd = RHT2F_TEST_AUTH_ATTEMPTS;
+    packet.cmd = HT2F_TEST_AUTH_ATTEMPTS;
 
     clearCommandBuffer();
     SendCommandNG(CMD_LF_HITAG_READER, (uint8_t *)&packet, sizeof(packet));
     PacketResponseNG resp;
     if (WaitForResponseTimeout(CMD_LF_HITAG_READER, &resp, 2000) == false) {
-        PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+        PrintAndLogEx(WARNING, "timeout while waiting for reply");
         return PM3_ETIMEOUT;
     }
     if (resp.status != PM3_SUCCESS) {
@@ -1163,17 +1158,13 @@ static int CmdLFHitag2CheckChallenges(const char *Cmd) {
 static int CmdLFHitagWriter(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "lf hitag wrbl",
-                  "Write a page in Hitag memory. It support HitagS and Hitag 2\n"
+                  "Write a page in Hitag memory. It support Hitag 2\n"
                   "  Password mode:\n"
                   "    - default key 4D494B52 (MIKR)\n\n"
                   "  Crypto mode: \n"
                   "    - key format ISK high + ISK low\n"
                   "    - default key 4F4E4D494B52 (ONMIKR)\n"
                   ,
-                  "  lf hitag wrbl --hts -p 6 -d 01020304                         -> HitagS, plain mode\n"
-                  "  lf hitag wrbl --hts -p 6 -d 01020304 --nrar 0102030411223344 -> HitagS, challenge mode\n"
-                  "  lf hitag wrbl --hts -p 6 -d 01020304 --crypto                -> HitagS, crypto mode, def key\n"
-                  "  lf hitag wrbl --hts -p 6 -d 01020304 -k 4F4E4D494B52         -> HitagS, crypto mode\n\n"
                   "  lf hitag wrbl --ht2 -p 6 -d 01020304 --pwd                   -> Hitag 2, pwd mode, def key\n"
                   "  lf hitag wrbl --ht2 -p 6 -d 01020304 -k 4D494B52             -> Hitag 2, pwd mode\n"
                   "  lf hitag wrbl --ht2 -p 6 -d 01020304 --nrar 0102030411223344 -> Hitag 2, challenge mode\n"
@@ -1183,7 +1174,6 @@ static int CmdLFHitagWriter(const char *Cmd) {
 
     void *argtable[] = {
         arg_param_begin,
-        arg_lit0("s", "hts", "Hitag S"),
         arg_lit0("2", "ht2", "Hitag 2"),
         arg_lit0(NULL, "pwd", "password mode"),
         arg_str0(NULL, "nrar", "<hex>", "nonce / answer writer, 8 hex bytes"),
@@ -1196,35 +1186,34 @@ static int CmdLFHitagWriter(const char *Cmd) {
     CLIExecWithReturn(ctx, Cmd, argtable, false);
 
     bool use_ht1 = false; // not yet implemented
-    bool use_hts = arg_get_lit(ctx, 1);
-    bool use_ht2 = arg_get_lit(ctx, 2);
+    bool use_ht2 = arg_get_lit(ctx, 1);
     bool use_htm = false; // not yet implemented
 
     bool use_plain = false;
-    bool use_pwd = arg_get_lit(ctx, 3);
+    bool use_pwd = arg_get_lit(ctx, 2);
     uint8_t nrar[8];
     int nalen = 0;
-    int res = CLIParamHexToBuf(arg_get_str(ctx, 4), nrar, sizeof(nrar), &nalen);
+    int res = CLIParamHexToBuf(arg_get_str(ctx, 3), nrar, sizeof(nrar), &nalen);
     if (res != 0) {
         CLIParserFree(ctx);
         return PM3_EINVARG;
     }
     bool use_nrar = nalen > 0;
-    bool use_crypto = arg_get_lit(ctx, 5);
+    bool use_crypto = arg_get_lit(ctx, 4);
 
     uint8_t key[6];
     int keylen = 0;
-    res = CLIParamHexToBuf(arg_get_str(ctx, 6), key, sizeof(key), &keylen);
+    res = CLIParamHexToBuf(arg_get_str(ctx, 5), key, sizeof(key), &keylen);
     if (res != 0) {
         CLIParserFree(ctx);
         return PM3_EINVARG;
     }
 
-    int page = arg_get_int_def(ctx, 7, 0);
+    int page = arg_get_int_def(ctx, 6, 0);
 
     uint8_t data[4];
     int dlen = 0;
-    res = CLIParamHexToBuf(arg_get_str(ctx, 8), data, sizeof(data), &dlen);
+    res = CLIParamHexToBuf(arg_get_str(ctx, 7), data, sizeof(data), &dlen);
     if (res != 0) {
         CLIParserFree(ctx);
         return PM3_EINVARG;
@@ -1233,11 +1222,11 @@ static int CmdLFHitagWriter(const char *Cmd) {
     CLIParserFree(ctx);
 
     // sanity checks
-    if ((use_ht1 + use_ht2 + use_hts + use_htm) > 1) {
+    if ((use_ht1 + use_ht2 + use_htm) > 1) {
         PrintAndLogEx(ERR, "error, specify only one Hitag type");
         return PM3_EINVARG;
     }
-    if ((use_ht1 + use_ht2 + use_hts + use_htm) == 0) {
+    if ((use_ht1 + use_ht2 + use_htm) == 0) {
         PrintAndLogEx(ERR, "error, specify one Hitag type");
         return PM3_EINVARG;
     }
@@ -1279,16 +1268,7 @@ static int CmdLFHitagWriter(const char *Cmd) {
         PrintAndLogEx(WARNING, "Specify only one authentication mode");
         return PM3_EINVARG;
     } else if (foo == 0) {
-        if (use_hts) {
-            use_plain = true;
-        } else {
-            PrintAndLogEx(WARNING, "Specify one authentication mode");
-            return PM3_EINVARG;
-        }
-    }
-
-    if (use_hts && use_pwd) { // not sure for the other types...
-        PrintAndLogEx(WARNING, "Chosen Hitag type does not have Password mode");
+        PrintAndLogEx(WARNING, "Specify one authentication mode");
         return PM3_EINVARG;
     }
 
@@ -1300,36 +1280,15 @@ static int CmdLFHitagWriter(const char *Cmd) {
     lf_hitag_data_t packet;
     memset(&packet, 0, sizeof(packet));
 
-    if (use_hts && use_plain) {
-        packet.cmd = WHTSF_PLAIN;
-        packet.page = page;
-        memcpy(packet.data, data, sizeof(data));
-
-        PrintAndLogEx(INFO, "Write to " _YELLOW_("Hitag S") " in Plain mode");
-
-    } else if (use_hts && use_nrar) {
-        packet.cmd = WHTSF_CHALLENGE;
-        memcpy(packet.NrAr, nrar, sizeof(packet.NrAr));
-        memcpy(packet.data, data, sizeof(data));
-        // iceman:  No page in Hitag S ?
-        PrintAndLogEx(INFO, "Authenticating to " _YELLOW_("Hitag S") " in Challenge mode");
-
-    } else if (use_hts && use_crypto) {
-        packet.cmd = WHTSF_KEY;
-        memcpy(packet.key, key, sizeof(packet.key));
-        memcpy(packet.data, data, sizeof(data));
-        // iceman:  No page in Hitag S ?
-        PrintAndLogEx(INFO, "Authenticating to " _YELLOW_("Hitag S") " in Crypto mode");
-
-    } else if (use_ht2 && use_pwd) {
-        packet.cmd = WHT2F_PASSWORD;
+    if (use_ht2 && use_pwd) {
+        packet.cmd = HT2F_PASSWORD;
         packet.page = page;
         memcpy(packet.pwd, key, sizeof(packet.pwd));
         memcpy(packet.data, data, sizeof(data));
         PrintAndLogEx(INFO, "Authenticating to " _YELLOW_("Hitag 2") " in Password mode");
 
     } else if (use_ht2 && use_crypto) {
-        packet.cmd = WHT2F_CRYPTO;
+        packet.cmd = HT2F_CRYPTO;
         packet.page = page;
         memcpy(packet.key, key, sizeof(packet.key));
         memcpy(packet.data, data, sizeof(data));
@@ -1346,7 +1305,7 @@ static int CmdLFHitagWriter(const char *Cmd) {
         SendCommandNG(CMD_LF_HITAG2_WRITE, (uint8_t *)&packet, sizeof(packet));
         PacketResponseNG resp;
         if (WaitForResponseTimeout(CMD_LF_HITAG2_WRITE, &resp, 4000) == false) {
-            PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+            PrintAndLogEx(WARNING, "timeout while waiting for reply");
             return PM3_ETIMEOUT;
         }
 
@@ -1360,24 +1319,6 @@ static int CmdLFHitagWriter(const char *Cmd) {
             return resp.status;
         }
 
-    } else {
-
-        SendCommandNG(CMD_LF_HITAGS_WRITE, (uint8_t *)&packet, sizeof(packet));
-        PacketResponseNG resp;
-        if (WaitForResponseTimeout(CMD_LF_HITAGS_WRITE, &resp, 4000) == false) {
-            PrintAndLogEx(WARNING, "timeout while waiting for reply.");
-            return PM3_ETIMEOUT;
-        }
-
-        if (resp.status == PM3_ETEAROFF) {
-            PrintAndLogEx(INFO, "Writing tear off triggered");
-            return PM3_SUCCESS;
-        }
-
-        if (resp.status != PM3_SUCCESS) {
-            PrintAndLogEx(FAILED, "Write ( " _RED_("fail") " )");
-            return resp.status;
-        }
     }
 
     PrintAndLogEx(SUCCESS, "Write ( " _GREEN_("ok") " )");
@@ -1515,12 +1456,12 @@ static int CmdLFHitag2Dump(const char *Cmd) {
     memset(&packet, 0, sizeof(packet));
 
     if (use_ht2 && use_pwd) {
-        packet.cmd = RHT2F_PASSWORD;
+        packet.cmd = HT2F_PASSWORD;
         memcpy(packet.pwd, key, sizeof(packet.pwd));
         PrintAndLogEx(INFO, "Authenticating to " _YELLOW_("Hitag 2") " in Password mode");
 
     } else if (use_ht2 && use_crypto) {
-        packet.cmd = RHT2F_CRYPTO;
+        packet.cmd = HT2F_CRYPTO;
         memcpy(packet.key, key, sizeof(packet.key));
         PrintAndLogEx(INFO, "Authenticating to " _YELLOW_("Hitag 2") " in Crypto mode");
 
@@ -1587,7 +1528,7 @@ static int CmdLFHitag2Dump(const char *Cmd) {
 
         if (attempt == 0) {
             PrintAndLogEx(NORMAL, "");
-            PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+            PrintAndLogEx(WARNING, "timeout while waiting for reply");
             return PM3_ESOFT;
         }
 
@@ -1605,7 +1546,7 @@ static int CmdLFHitag2Dump(const char *Cmd) {
     SendCommandNG(CMD_LF_HITAG_READER, (uint8_t *) &packet, sizeof(packet));
 
     if (WaitForResponseTimeout(CMD_LF_HITAG_READER, &resp, 5000) == false) {
-        PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+        PrintAndLogEx(WARNING, "timeout while waiting for reply");
         return PM3_ETIMEOUT;
     }
     if (resp.status != PM3_SUCCESS) {
@@ -1624,7 +1565,7 @@ out:
     if (use_ht2) {
         print_hitag2_configuration(uid, data[HITAG_BLOCK_SIZE * 3]);
         print_hitag2_blocks(data, HITAG2_MAX_BYTE_SIZE);
-        print_hitag2_paxton(data);
+        print_hitag2_paxton(true, data);
     } else {
         PrintAndLogEx(INFO, "No memory printing available");
     }
@@ -1685,7 +1626,7 @@ static int CmdLFHitagView(const char *Cmd) {
         uint8_t config = dump[HITAG2_CONFIG_OFFSET];
         uint32_t uid = bytes_to_num(dump, HITAG_UID_SIZE);
         print_hitag2_configuration(uid, config);
-        print_hitag2_paxton(dump);
+        print_hitag2_paxton(true, dump);
     }
     print_hitag2_blocks(dump, HITAG2_MAX_BYTE_SIZE);
     free(dump);
@@ -1739,7 +1680,12 @@ static int CmdLFHitagEload(const char *Cmd) {
     // check dump len..
     if (bytes_read == HITAG2_MAX_BYTE_SIZE || bytes_read == 4 * 64) {
 
-        lf_hitag_t *payload =  calloc(1, sizeof(lf_hitag_t) + bytes_read);
+        lf_hitag_t *payload = calloc(1, sizeof(lf_hitag_t) + bytes_read);
+        if (payload == NULL) {
+            PrintAndLogEx(WARNING, "Failed to allocate memory");
+            free(dump);
+            return PM3_EMALLOC;
+        }
 
         if (use_ht1)
             payload->type = 1;
@@ -1784,7 +1730,7 @@ static int CmdLFHitagEview(const char *Cmd) {
     // reserve memory
     uint8_t *dump = calloc(bytes, sizeof(uint8_t));
     if (dump == NULL) {
-        PrintAndLogEx(WARNING, "Fail, cannot allocate memory");
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return PM3_EMALLOC;
     }
 
@@ -1800,7 +1746,7 @@ static int CmdLFHitagEview(const char *Cmd) {
         uint8_t config = dump[HITAG2_CONFIG_OFFSET];
         uint32_t uid = bytes_to_num(dump, HITAG_UID_SIZE);
         print_hitag2_configuration(uid, config);
-        print_hitag2_paxton(dump);
+        print_hitag2_paxton(true, dump);
     }
     print_hitag2_blocks(dump, HITAG2_MAX_BYTE_SIZE);
     free(dump);
@@ -1819,22 +1765,20 @@ static int CmdLFHitagSim(const char *Cmd) {
         arg_param_begin,
         arg_lit0("1", "ht1", "simulate Hitag 1"),
         arg_lit0("2", "ht2", "simulate Hitag 2"),
-        arg_lit0("s", "hts", "simulate Hitag S"),
         arg_param_end
     };
     CLIExecWithReturn(ctx, Cmd, argtable, true);
 
     bool use_ht1 = arg_get_lit(ctx, 1);
     bool use_ht2 = arg_get_lit(ctx, 2);
-    bool use_hts = arg_get_lit(ctx, 3);
     bool use_htm = false; // not implemented yet
     CLIParserFree(ctx);
 
-    if ((use_ht1 + use_ht2 + use_hts + use_htm) > 1) {
+    if ((use_ht1 + use_ht2 + use_htm) > 1) {
         PrintAndLogEx(ERR, "error, specify only one Hitag type");
         return PM3_EINVARG;
     }
-    if ((use_ht1 + use_ht2 + use_hts + use_htm) == 0) {
+    if ((use_ht1 + use_ht2 + use_htm) == 0) {
         PrintAndLogEx(ERR, "error, specify one Hitag type");
         return PM3_EINVARG;
     }
@@ -1843,9 +1787,6 @@ static int CmdLFHitagSim(const char *Cmd) {
 //    if (use_ht1)
 //        cmd = CMD_LF_HITAG1_SIMULATE;
 
-    if (use_hts)
-        cmd = CMD_LF_HITAGS_SIMULATE;
-
     clearCommandBuffer();
     SendCommandMIX(cmd, 0, 0, 0, NULL, 0);
     return PM3_SUCCESS;
@@ -1854,7 +1795,7 @@ static int CmdLFHitagSim(const char *Cmd) {
 static int CmdLFHitagSniff(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "lf hitag sniff",
-                  "Sniff the communication between reader and tag.\n"
+                  "Sniff the communication between reader and tag\n"
                   "Use `lf hitag list` to view collected data.",
                   " lf hitag sniff"
                  );
@@ -1873,8 +1814,8 @@ static int CmdLFHitagSniff(const char *Cmd) {
     SendCommandNG(CMD_LF_HITAG_SNIFF, NULL, 0);
     WaitForResponse(CMD_LF_HITAG_SNIFF, &resp);
     PrintAndLogEx(INFO, "Done!");
-    PrintAndLogEx(HINT, "Try `" _YELLOW_("lf hitag list")"` to view captured tracelog");
-    PrintAndLogEx(HINT, "Try `" _YELLOW_("trace save -h") "` to save tracelog for later analysing");
+    PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("lf hitag list")"` to view captured tracelog");
+    PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("trace save -h") "` to save tracelog for later analysing");
     return PM3_SUCCESS;
 }
 
@@ -1904,7 +1845,7 @@ static int CmdLFHitag2PWMDemod(const char *Cmd) {
 
     uint8_t *bits = calloc(MAX_GRAPH_TRACE_LEN, sizeof(uint8_t));
     if (bits == NULL) {
-        PrintAndLogEx(INFO, "failed to allocate memory");
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return PM3_EMALLOC;
     }
 
@@ -1952,7 +1893,7 @@ static int CmdLFHitag2Chk(const char *Cmd) {
     CLIParserContext *ctx;
     CLIParserInit(&ctx, "lf hitag chk",
                   "Run dictionary key or password recovery against Hitag card.",
-                  "lf hitag chk\n               -> checks for both pwd / crypto keys"
+                  "lf hitag chk                 -> checks for both pwd / crypto keys\n"
                   "lf hitag chk --crypto        -> use def dictionary\n"
                   "lf hitag chk --pwd -f my.dic -> pwd mode, custom dictionary"
                  );
@@ -2021,6 +1962,12 @@ static int CmdLFHitag2Chk(const char *Cmd) {
             break;
         }
         free(keys);
+
+        PrintAndLogEx(NORMAL, "");
+
+        if (use_pwd) {
+            break;
+        }
     }
 
     t1 = msclock() - t1;
@@ -2302,7 +2249,7 @@ static int CmdLFHitag2Crack2(const char *Cmd) {
             }
             PrintAndLogEx(NORMAL, "");
             PrintAndLogEx(SUCCESS, "Nonce replay and length extension attack ( %s )", _GREEN_("ok"));
-            PrintAndLogEx(HINT, "try running `tools/hitag2crack/crack2/ht2crack2search <FILE_with_above_bytes>");
+            PrintAndLogEx(HINT, "Hint: Try `" _YELLOW_("tools/hitag2crack/crack2/ht2crack2search <FILE_with_above_bytes>") "`");
             break;
         } else {
             PrintAndLogEx(NORMAL, "");
@@ -2314,7 +2261,7 @@ static int CmdLFHitag2Crack2(const char *Cmd) {
 
     if (attempt == 0) {
         PrintAndLogEx(NORMAL, "");
-        PrintAndLogEx(WARNING, "timeout while waiting for reply.");
+        PrintAndLogEx(WARNING, "timeout while waiting for reply");
         return PM3_ESOFT;
     }
 
@@ -2385,9 +2332,9 @@ static uint64_t hitag2_verify_crypto_test(void) {
     // key = 0x4ad292b272f2  after each byte has its bit order reversed
     // uid = 0x96eac292      ditto
     // initvec = 0x4ea276a6  ditto
-    const uint64_t key = REV64(0x524B494D4E4FUL);
-    const uint32_t uid = REV32(0x69574349);
-    const uint32_t iv = REV32(0x72456E65);
+    const uint64_t key = REV64(UINT64_C(0x524B494D4E4F));
+    const uint32_t uid = REV32(UINT32_C(0x69574349));
+    const uint32_t iv = REV32(UINT32_C(0x72456E65));
 
     PrintAndLogEx(DEBUG, "UID... %08" PRIx32, uid);
     PrintAndLogEx(DEBUG, "IV.... %08" PRIx32, iv);
@@ -2414,9 +2361,9 @@ static uint64_t hitag2_verify_crypto_test(void) {
 static uint64_t hitag2_verify_crypto_test_round(void) {
 
     uint8_t expected[16] = { 0xD7, 0x23, 0x7F, 0xCE, 0x8C, 0xD0, 0x37, 0xA9, 0x57, 0x49, 0xC1, 0xE6, 0x48, 0x00, 0x8A, 0xB6 };
-    const uint64_t key = REV64(0x524B494D4E4FUL);
-    const uint32_t uid = REV32(0x69574349);
-    const uint32_t iv = REV32(0x72456E65);
+    const uint64_t key = REV64(UINT64_C(0x524B494D4E4F));
+    const uint32_t uid = REV32(UINT32_C(0x69574349));
+    const uint32_t iv = REV32(UINT32_C(0x72456E65));
 
     PrintAndLogEx(DEBUG, "UID... %08" PRIx32, uid);
     PrintAndLogEx(DEBUG, "IV.... %08" PRIx32, iv);
@@ -2475,12 +2422,53 @@ static int CmdLFHitag2Selftest(const char *Cmd) {
     return PM3_SUCCESS;
 }
 
+int ht2_read_uid(void) {
+    uint32_t uid = 0;
+    if (ht2_get_uid(&uid) == false) {
+        return PM3_ESOFT;
+    }
+
+    PrintAndLogEx(SUCCESS, "UID.... " _GREEN_("%08X"), uid);
+    PrintAndLogEx(SUCCESS, "TYPE... " _GREEN_("%s"), getHitagTypeStr(uid));
+    return PM3_SUCCESS;
+}
+
+int ht2_read_paxton(void) {
+
+// read block 4,5,6,7
+
+    lf_hitag_data_t packet;
+    memset(&packet, 0, sizeof(packet));
+
+    packet.cmd = HT2F_PASSWORD;
+    memcpy(packet.pwd, ht2_default_keys, sizeof(packet.pwd));
+
+    clearCommandBuffer();
+    SendCommandNG(CMD_LF_HITAG_READER, (uint8_t *)&packet, sizeof(packet));
+
+    PacketResponseNG resp;
+    if (WaitForResponseTimeout(CMD_LF_HITAG_READER, &resp, 2000) == false) {
+        SendCommandNG(CMD_BREAK_LOOP, NULL, 0);
+        return PM3_ETIMEOUT;
+    }
+
+    if (resp.status != PM3_SUCCESS) {
+        PrintAndLogEx(DEBUG, "DEBUG: Error - hitag failed");
+        return PM3_ESOFT;
+    }
+
+    uint8_t *data = resp.data.asBytes;
+    print_hitag2_paxton(false, data);
+    return PM3_SUCCESS;
+}
+
 static command_t CommandTable[] = {
     {"help",        CmdHelp,                    AlwaysAvailable, "This help"},
     {"list",        CmdLFHitagList,             AlwaysAvailable, "List Hitag trace history"},
-    {"hts",           CmdLFHitagS,                AlwaysAvailable, "{ Hitag S/8211 operations }"},
+    {"hts",         CmdLFHitagS,                AlwaysAvailable, "{ Hitag S/8211 operations }"},
+    {"htu",         CmdLFHitagU,                AlwaysAvailable, "{ Hitag µ/8265 operations }"},
     {"-----------", CmdHelp,                    IfPm3Hitag,      "------------------------ " _CYAN_("General") " ------------------------"},
-    {"info",        CmdLFHitagInfo,             IfPm3Hitag,      "Hitag 2 tag information"},
+    {"info",        CmdLFHitagInfo,             IfPm3Hitag,      "Tag information"},
     {"reader",      CmdLFHitagReader,           IfPm3Hitag,      "Act like a Hitag 2 reader"},
     {"test",        CmdLFHitag2Selftest,        AlwaysAvailable, "Perform self tests"},
     {"-----------", CmdHelp,                    IfPm3Hitag,      "----------------------- " _CYAN_("Operations") " -----------------------"},
@@ -2515,14 +2503,4 @@ int CmdLFHitag(const char *Cmd) {
     return CmdsParse(CommandTable, Cmd);
 }
 
-int readHitagUid(void) {
-    uint32_t uid = 0;
-    if (getHitag2Uid(&uid) == false) {
-        return PM3_ESOFT;
-    }
-
-    PrintAndLogEx(SUCCESS, "UID.... " _GREEN_("%08X"), uid);
-    PrintAndLogEx(SUCCESS, "TYPE... " _GREEN_("%s"), getHitagTypeStr(uid));
-    return PM3_SUCCESS;
-}
 
