@@ -106,6 +106,10 @@ void DesfireSetKdf(DesfireContext_t *ctx, uint8_t kdfAlgo, uint8_t *kdfInput, ui
     }
 }
 
+void DesfireSetSecureChannel(DesfireContext_t *ctx, DesfireSecureChannel schann) {
+    ctx->secureChannel = schann;
+}
+
 bool DesfireIsAuthenticated(DesfireContext_t *dctx) {
     return dctx->secureChannel != DACNone;
 }
@@ -503,18 +507,22 @@ uint8_t DesfireKeyAlgoToType(DesfireCryptoAlgorithm keyType) {
 
 void DesfirePrintCardKeyType(uint8_t keyType) {
     switch (keyType) {
-        case 00:
-            PrintAndLogEx(SUCCESS, "Key: 2TDEA");
+        case 00: {
+            PrintAndLogEx(SUCCESS, "Key type... " _YELLOW_("2TDEA"));
             break;
-        case 01:
-            PrintAndLogEx(SUCCESS, "Key: 3TDEA");
+        }
+        case 01: {
+            PrintAndLogEx(SUCCESS, "Key type... " _YELLOW_("3TDEA"));
             break;
-        case 02:
-            PrintAndLogEx(SUCCESS, "Key: AES");
+        }
+        case 02: {
+            PrintAndLogEx(SUCCESS, "Key type... " _YELLOW_("AES"));
             break;
-        default:
-            PrintAndLogEx(SUCCESS, "Key: unknown: 0x%02x", keyType);
+        }
+        default: {
+            PrintAndLogEx(SUCCESS, "Key type... " _YELLOW_("unknown") " - 0x%02x", keyType);
             break;
+        }
     }
 }
 
