@@ -69,8 +69,13 @@ static int open_mad_file(json_t **root, bool verbose) {
         goto out;
     }
 
-    if (verbose)
-        PrintAndLogEx(SUCCESS, "Loaded file " _YELLOW_("`%s`") " (%s) %zu records.", path,  _GREEN_("ok"), json_array_size(*root));
+    if (verbose) {
+        PrintAndLogEx(SUCCESS, "Loaded file `" _YELLOW_("%s") "` " _GREEN_("%zu") " records ( " _GREEN_("ok") " )"
+                      , path
+                      , json_array_size(*root)
+                     );
+    }
+
 out:
     free(path);
     return retval;
@@ -140,15 +145,19 @@ static int print_aid_description(json_t *root, uint16_t aid, char *fmt, bool ver
     }
 
     if (verbose) {
-        PrintAndLogEx(SUCCESS, "    MAD:               %s", vmad);
-        if (application)
-            PrintAndLogEx(SUCCESS, "    Application:       %s", application);
-        if (company)
-            PrintAndLogEx(SUCCESS, "    Company:           %s", company);
-        if (provider)
-            PrintAndLogEx(SUCCESS, "    Service provider:  %s", provider);
-        if (integrator)
-            PrintAndLogEx(SUCCESS, "    System integrator: %s", integrator);
+        PrintAndLogEx(SUCCESS, "     MAD................. %s", vmad);
+        if (application) {
+            PrintAndLogEx(SUCCESS, "     Application......... %s", application);
+        }
+        if (company) {
+            PrintAndLogEx(SUCCESS, "     Company............. %s", company);
+        }
+        if (provider) {
+            PrintAndLogEx(SUCCESS, "     Service provider.... %s", provider);
+        }
+        if (integrator) {
+            PrintAndLogEx(SUCCESS, "     System integrator... %s", integrator);
+        }
     }
     return PM3_SUCCESS;
 }
@@ -183,7 +192,7 @@ static uint16_t madGetAID(const uint8_t *sector, bool swapmad, int MADver, int s
     }
 }
 
-int MADCheck(uint8_t *sector0, uint8_t *sector10, bool verbose, bool *haveMAD2) {
+int MADCheck(uint8_t *sector0, uint8_t *sector16, bool verbose, bool *haveMAD2) {
 
     if (sector0 == NULL)
         return PM3_EINVARG;
@@ -217,13 +226,13 @@ int MADCheck(uint8_t *sector0, uint8_t *sector10, bool verbose, bool *haveMAD2)
         PrintAndLogEx(SUCCESS, "CRC8...... 0x%02X ( %s )", sector0[16], _GREEN_("ok"));
     }
 
-    if (mad_ver == 2 && sector10) {
-        int res2 = madCRCCheck(sector10, true, 2);
+    if (mad_ver == 2 && sector16) {
+        int res2 = madCRCCheck(sector16, true, 2);
         if (res == PM3_SUCCESS)
             res = res2;
 
         if (verbose && !res2)
-            PrintAndLogEx(SUCCESS, "CRC8...... 0x%02X ( %s )", sector10[0], _GREEN_("ok"));
+            PrintAndLogEx(SUCCESS, "CRC8...... 0x%02X ( %s )", sector16[0], _GREEN_("ok"));
     }
 
     // MA (multi-application card)
@@ -236,27 +245,30 @@ int MADCheck(uint8_t *sector0, uint8_t *sector10, bool verbose, bool *haveMAD2)
     return res;
 }
 
-int MADDecode(uint8_t *sector0, uint8_t *sector10, uint16_t *mad, size_t *madlen, bool swapmad) {
+int MADDecode(uint8_t *sector0, uint8_t *sector16, uint16_t *mad, size_t *madlen, bool swapmad) {
     *madlen = 0;
     bool haveMAD2 = false;
-    int res = MADCheck(sector0, sector10, false, &haveMAD2);
+    int res = MADCheck(sector0, sector16, false, &haveMAD2);
     if (res != PM3_SUCCESS) {
         PrintAndLogEx(WARNING, "Not a valid MAD");
         return res;
     }
 
-    for (int i = 1; i < 16; i++) {
+    // 7 + 8 == 15
+    for (int i = 1; i <= 16; i++) {
         mad[*madlen] = madGetAID(sector0, swapmad, 1, i);
         (*madlen)++;
     }
 
     if (haveMAD2) {
-        // mad2 sector (0x10 == 16dec) here
+        // mad2 sector (0x10 == 16) here
         mad[*madlen] = 0x0005;
         (*madlen)++;
 
+        // 7 + 8 + 8  == 23
         for (int i = 1; i < 24; i++) {
-            mad[*madlen] = madGetAID(sector10, swapmad, 2, i);
+            mad[*madlen] = madGetAID(sector16, swapmad, 2, i);
+
             (*madlen)++;
         }
     }
@@ -347,8 +359,16 @@ int MAD1DecodeAndPrint(uint8_t *sector, bool swapmad, bool verbose, bool *haveMA
                           aid
                          );
         } else {
-            char fmt[60];
-            snprintf(fmt, sizeof(fmt), (ibs == i) ? _MAGENTA_(" %02d [%04X]%s") : " %02d [" _GREEN_("%04X") "]%s", i, aid, "%s");
+            char fmt[80];
+            snprintf(fmt
+                     , sizeof(fmt)
+                     , (ibs == i) ?
+                     _MAGENTA_(" %02d [%04X] %s") :
+                     " %02d [" _GREEN_("%04X") "] %s"
+                     , i
+                     , aid
+                     , "%s"
+                    );
             print_aid_description(mad_known_aids, aid, fmt, verbose);
             prev_aid = aid;
         }
@@ -388,20 +408,28 @@ int MAD2DecodeAndPrint(uint8_t *sector, bool swapmad, bool verbose) {
         uint16_t aid = madGetAID(sector, swapmad, 2, i);
         if (aid < 6) {
             PrintAndLogEx(INFO,
-                          (ibs == i) ? _MAGENTA_(" %02d [%04X] %s") : " %02d [" _GREEN_("%04X") "] %s",
+                          (ibs == i + 16) ? _MAGENTA_(" %02d [%04X] %s") : " %02d [" _GREEN_("%04X") "] %s",
                           i + 16,
                           aid,
                           aid_admin[aid]
                          );
         } else if (prev_aid == aid) {
             PrintAndLogEx(INFO,
-                          (ibs == i) ? _MAGENTA_(" %02d [%04X] continuation") : " %02d [" _YELLOW_("%04X") "] continuation",
+                          (ibs == i + 16) ? _MAGENTA_(" %02d [%04X] continuation") : " %02d [" _YELLOW_("%04X") "] continuation",
                           i + 16,
                           aid
                          );
         } else {
-            char fmt[60];
-            snprintf(fmt, sizeof(fmt), (ibs == i) ? _MAGENTA_(" %02d [%04X]%s") : " %02d [" _GREEN_("%04X") "]%s", i + 16, aid, "%s");
+            char fmt[80];
+            snprintf(fmt
+                     , sizeof(fmt)
+                     , (ibs == i + 16) ?
+                     _MAGENTA_(" %02d [%04X] %s") :
+                     " %02d [" _GREEN_("%04X") "] %s"
+                     , i + 16
+                     , aid
+                     , "%s"
+                    );
             print_aid_description(mad_known_aids, aid, fmt, verbose);
             prev_aid = aid;
         }
@@ -415,7 +443,7 @@ int MADDFDecodeAndPrint(uint32_t short_aid, bool verbose) {
     open_mad_file(&mad_known_aids, false);
 
     char fmt[128];
-    snprintf(fmt, sizeof(fmt), "  MAD AID Function 0x%04X    :" _YELLOW_("%s"), short_aid, "%s");
+    snprintf(fmt, sizeof(fmt), "   MAD AID Function 0x%04X... " _YELLOW_("%s"), short_aid, "%s");
     print_aid_description(mad_known_aids, short_aid, fmt, verbose);
     close_mad_file(mad_known_aids);
     return PM3_SUCCESS;
@@ -429,8 +457,9 @@ bool HasMADKey(uint8_t *d) {
 }
 
 int DetectHID(uint8_t *d, uint16_t manufacture) {
-    if (d == NULL)
+    if (d == NULL) {
         return -1;
+    }
 
     // find HID
     for (int i = 1; i < 16; i++) {
@@ -456,16 +485,16 @@ int convert_mad_to_arr(uint8_t *in, uint16_t ilen, uint8_t *out, uint16_t *olen)
     }
 
     uint8_t sector0[MFBLOCK_SIZE * 4] = {0};
-    uint8_t sector10[MFBLOCK_SIZE * 4] = {0};
+    uint8_t sector16[MFBLOCK_SIZE * 4] = {0};
 
     memcpy(sector0, in, sizeof(sector0));
     if (ilen == MIFARE_4K_MAX_BYTES) {
-        memcpy(sector10, in + (MF_MAD2_SECTOR * 4 * MFBLOCK_SIZE), sizeof(sector10));
+        memcpy(sector16, in + (MF_MAD2_SECTOR * 4 * MFBLOCK_SIZE), sizeof(sector16));
     }
 
     uint16_t mad[7 + 8 + 8 + 8 + 8] = {0};
     size_t madlen = 0;
-    if (MADDecode(sector0, sector10, mad, &madlen, false)) {
+    if (MADDecode(sector0, sector16, mad, &madlen, false)) {
         PrintAndLogEx(ERR, "can't decode MAD");
         return PM3_ESOFT;
     }
