@@ -63,7 +63,7 @@ const static vocabulary_t vocabulary[] = {
     { 1, "prefs set output" },
     { 1, "prefs set plotsliders" },
     { 1, "analyse help" },
-    { 1, "analyse lcr" },
+    { 1, "analyse lrc" },
     { 1, "analyse crc" },
     { 1, "analyse chksum" },
     { 1, "analyse dates" },
@@ -136,6 +136,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "emv scan" },
     { 0, "emv search" },
     { 0, "emv select" },
+    { 0, "emv smart2nfc" },
     { 1, "hf help" },
     { 1, "hf list" },
     { 0, "hf plot" },
@@ -149,6 +150,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf 14a cuids" },
     { 0, "hf 14a info" },
     { 0, "hf 14a sim" },
+    { 0, "hf 14a simaid" },
     { 0, "hf 14a sniff" },
     { 0, "hf 14a raw" },
     { 0, "hf 14a reader" },
@@ -175,6 +177,7 @@ const static vocabulary_t vocabulary[] = {
     { 1, "hf 14b valid" },
     { 0, "hf 14b calypso" },
     { 0, "hf 14b mobib" },
+    { 0, "hf 14b setuid" },
     { 1, "hf 15 help" },
     { 1, "hf 15 list" },
     { 1, "hf 15 demod" },
@@ -199,6 +202,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf 15 slixeasenable" },
     { 0, "hf 15 slixprivacydisable" },
     { 0, "hf 15 slixprivacyenable" },
+    { 0, "hf 15 slixprotectpage" },
     { 0, "hf 15 passprotectafi" },
     { 0, "hf 15 passprotecteas" },
     { 0, "hf 15 findafi" },
@@ -265,6 +269,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf gallagher delete" },
     { 1, "hf gallagher diversifykey" },
     { 1, "hf gallagher decode" },
+    { 1, "hf gallagher encode" },
     { 1, "hf iclass help" },
     { 1, "hf iclass list" },
     { 0, "hf iclass dump" },
@@ -276,11 +281,13 @@ const static vocabulary_t vocabulary[] = {
     { 1, "hf iclass view" },
     { 0, "hf iclass wrbl" },
     { 0, "hf iclass creditepurse" },
+    { 0, "hf iclass tear" },
     { 0, "hf iclass chk" },
     { 1, "hf iclass loclass" },
     { 1, "hf iclass lookup" },
     { 0, "hf iclass legrec" },
     { 1, "hf iclass legbrute" },
+    { 1, "hf iclass unhash" },
     { 0, "hf iclass sim" },
     { 0, "hf iclass eload" },
     { 0, "hf iclass esave" },
@@ -336,8 +343,6 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf lto wrbl" },
     { 1, "hf mf help" },
     { 1, "hf mf list" },
-    { 0, "hf mf info" },
-    { 0, "hf mf isen" },
     { 0, "hf mf darkside" },
     { 0, "hf mf nested" },
     { 1, "hf mf hardnested" },
@@ -349,9 +354,12 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf mf fchk" },
     { 1, "hf mf decrypt" },
     { 0, "hf mf supercard" },
+    { 1, "hf mf bambukeys" },
     { 0, "hf mf auth4" },
     { 1, "hf mf acl" },
     { 0, "hf mf dump" },
+    { 0, "hf mf info" },
+    { 0, "hf mf isen" },
     { 1, "hf mf mad" },
     { 0, "hf mf personalize" },
     { 0, "hf mf rdbl" },
@@ -383,7 +391,7 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf mf gen3uid" },
     { 0, "hf mf gen3blk" },
     { 0, "hf mf gen3freeze" },
-    { 0, "hf mf ginfo" },
+    { 1, "hf mf ginfo" },
     { 0, "hf mf ggetblk" },
     { 0, "hf mf gload" },
     { 0, "hf mf gsave" },
@@ -423,7 +431,9 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf mfu otptear" },
     { 0, "hf mfu cauth" },
     { 0, "hf mfu setpwd" },
+    { 0, "hf mfu aesauth" },
     { 0, "hf mfu dump" },
+    { 0, "hf mfu incr" },
     { 0, "hf mfu info" },
     { 0, "hf mfu ndefread" },
     { 0, "hf mfu rdbl" },
@@ -487,8 +497,13 @@ const static vocabulary_t vocabulary[] = {
     { 0, "hf ntag424 changefs" },
     { 0, "hf ntag424 changekey" },
     { 1, "hf seos help" },
-    { 0, "hf seos info" },
     { 1, "hf seos list" },
+    { 0, "hf seos sam" },
+    { 0, "hf seos info" },
+    { 1, "hf seos pacs" },
+    { 1, "hf seos adf" },
+    { 1, "hf seos gdf" },
+    { 1, "hf seos managekeys" },
     { 1, "hf st25ta help" },
     { 0, "hf st25ta info" },
     { 1, "hf st25ta list" },
@@ -671,8 +686,19 @@ const static vocabulary_t vocabulary[] = {
     { 0, "lf hitag ta" },
     { 1, "lf hitag hts help" },
     { 1, "lf hitag hts list" },
-    { 0, "lf hitag hts read" },
-    { 0, "lf hitag hts write" },
+    { 0, "lf hitag hts reader" },
+    { 0, "lf hitag hts rdbl" },
+    { 0, "lf hitag hts dump" },
+    { 0, "lf hitag hts restore" },
+    { 0, "lf hitag hts wrbl" },
+    { 0, "lf hitag hts sim" },
+    { 1, "lf hitag htu help" },
+    { 1, "lf hitag htu list" },
+    { 0, "lf hitag htu reader" },
+    { 0, "lf hitag htu rdbl" },
+    { 0, "lf hitag htu dump" },
+    { 0, "lf hitag htu wrbl" },
+    { 0, "lf hitag htu sim" },
     { 1, "lf idteck help" },
     { 1, "lf idteck demod" },
     { 0, "lf idteck reader" },
