@@ -48,8 +48,8 @@ static int mainret = PM3_SUCCESS;
 
 #ifndef LIBPM3
 #define BANNERMSG1 ""
-#define BANNERMSG2 "   [ :coffee: ]"
-#define BANNERMSG3 "Release v4.18994 - Backdoor"
+#define BANNERMSG2 ""
+#define BANNERMSG3 "Release v4.20469 - Daddy Iceman"
 
 typedef enum LogoMode { UTF8, ANSI, ASCII } LogoMode;
 
@@ -125,83 +125,65 @@ static uint8_t detect_current_lang(void) {
 static const char *get_quote(void) {
 
     const char *quotes_en[] = {
-        "Fund creativity, empower dreams",
-        "Invest in open innovation",
-        "Donate, empower, grow, sustain",
-        "Back global innovation today",
-        "Fuel open source revolution",
-        "Contribute funds, drive progress",
-        "Sponsor innovation, build tomorrow",
-        "Consider supporting: fund innovation",
-        "Your donation fuels progress",
-        "Empower dreams with your support",
-        "Join us: finance creative freedom",
-        "Make an impact: donate today",
-        "Help us drive open innovation",
-        "Your support, our future",
-        "Invest in a better tomorrow",
-        "Every contribution powers change",
-        "Support us, shape the future",
-        "Ignite change: support open-source creativity",
-        "Together, we can innovate without limits",
+        "too many secrets",
+        "It's not that simple",
+        "I have received a coded signal",
+        "I await your instructions",
+        "And so I watch, I wait",
+        "Listen to the Domain",
+        "ghost.713",
+        "Local node X.XX.713",
+        "Beggar after knowledge",
+        "343 Gulity Spark: offline",
+        "I serve the Builders!",
+        "This is rather distressing"
     };
 
     const char *quotes_fr[] = {
-        "Financez la créativité, donnez pouvoir aux rêves",
-        "Investissez dans l'innovation ouverte",
-        "Donnez, habilitez, croissez, soutenez",
-        "Soutenez l'innovation mondiale aujourd'hui",
-        "Alimentez la révolution open source",
-        "Contribuez financièrement, poussez le progrès",
-        "Parrainez l'innovation, construisez demain",
-        "Envisagez de soutenir : financez l'innovation",
-        "Votre don alimente le progrès",
-        "Donnez pouvoir aux rêves avec votre soutien",
-        "Rejoignez-nous : financez la liberté créative",
-        "Faites une différence : donnez aujourd'hui",
-        "Aidez-nous à stimuler l'innovation ouverte",
-        "Votre soutien, notre avenir",
-        "Investissez dans un meilleur demain",
-        "Chaque contribution favorise le changement",
-        "Soutenez-nous, façonnez l'avenir",
-        "Allumez le changement : soutenez la créativité open-source",
-        "Ensemble, nous pouvons innover sans limites",
+        "Liberté, égalité, fraternité",
+        "L'avenir appartient à ceux qui croient à la beauté de leurs rêves",
+        "Rien n'est impossible",
+        "La vie est un défi, relève-le!",
+        "Qui ne tente rien n'a rien",
+        "Le succès est la somme de petits efforts, répétés jour après jour",
+        "Faites de votre vie un rêve, et d'un rêve, une réalité",
+        "La seule façon de faire du bon travail est d'aimer ce que vous faites",
+        "Tout ce que vous pouvez faire, faites-le",
+        "Le succès, c'est d'aller d'échec en échec sans perdre son enthousiasme",
+        "Crois en toi et tout deviendra possible",
+        "C'est en tombant qu'on apprend à se relever"
     };
 
     const char *quotes_es[] = {
-        "Financia la creatividad, empodera sueños",
-        "Invierte en innovación abierta",
-        "Dona, empodera, crece, sostén",
-        "Apoya la innovación global hoy",
-        "Impulsa la revolución de código abierto",
-        "Contribuye fondos, impulsa el progreso",
-        "Patrocina la innovación, construye el mañana",
-        "Considera apoyar: financia la innovación",
-        "Tu donación impulsa el progreso",
-        "Empodera sueños con tu apoyo",
-        "Únete a nosotros: financia la libertad creativa",
-        "Haz un impacto: dona hoy",
-        "Ayúdanos a impulsar la innovación abierta",
-        "Tu apoyo, nuestro futuro",
-        "Invierte en un mejor mañana",
-        "Cada contribución impulsa el cambio",
-        "Apóyanos, forma el futuro",
-        "Enciende el cambio: apoya la creatividad de código abierto",
-        "Juntos, podemos innovar sin límites",
+        "El éxito es la suma de pequeños esfuerzos repetidos día tras día",
+        "Hazlo con pasión o no lo hagas",
+        "Nunca dejes de soñar",
+        "El único modo de hacer un gran trabajo es amar lo que haces",
+        "No hay que ir para atrás ni para darse impulso",
+        "Cada logro comienza con la decisión de intentarlo",
+        "No importa lo lento que vayas, siempre y cuando no te detengas",
+        "La disciplina es el puente entre las metas y los logros",
+        "Si puedes soñarlo, puedes lograrlo",
+        "La vida es una aventura, atrévete",
     };
 
+    int r = 0;
     srand((uint32_t)time(NULL));
-    int r = rand() % ARRAYLEN(quotes_en);
-
     uint8_t lang = detect_current_lang();
     switch (lang) {
-        case 2:
+        case 2: {
+            r = rand() % ARRAYLEN(quotes_fr);
             return quotes_fr[r];
-        case 3:
+        }
+        case 3: {
+            r = rand() % ARRAYLEN(quotes_es);
             return quotes_es[r];
+        }
         case 1:
-        default:
+        default: {
+            r = rand() % ARRAYLEN(quotes_en);
             return quotes_en[r];
+        }
     }
 }
 
@@ -223,11 +205,9 @@ static void showBanner(void) {
     showBanner_logo(ASCII);
 #endif
 
-    PrintAndLogEx(NORMAL, "");
-    PrintAndLogEx(NORMAL, "  [ " _YELLOW_("%s!")" ]", get_quote());
-    PrintAndLogEx(NORMAL, "     Patreon - https://www.patreon.com/iceman1001/");
-    PrintAndLogEx(NORMAL, "     Paypal  - https://www.paypal.me/iceman1001/");
-    PrintAndLogEx(NORMAL, "");
+    PrintAndLogEx(NORMAL, "  [ " _YELLOW_("%s!")" :coffee: ]", get_quote());
+//    PrintAndLogEx(NORMAL, "  [ https://patreon.com/iceman1001/ ]");
+//    PrintAndLogEx(NORMAL, "");
 //    PrintAndLogEx(NORMAL, "   Monero");
 //    PrintAndLogEx(NORMAL, " 43mNJLpgBVaTvyZmX9ajcohpvVkaRy1kbZPm8tqAb7itZgfuYecgkRF36rXrKFUkwEGeZedPsASRxgv4HPBHvJwyJdyvQuP");
     PrintAndLogEx(NORMAL, "");
@@ -278,7 +258,7 @@ static void prompt_compose(char *buf, size_t buflen, const char *promptctx, cons
     if (no_newline) {
         snprintf(buf, buflen - 1, PROXPROMPT_COMPOSE, promptdev, promptnet, promptctx);
     } else {
-        snprintf(buf, buflen - 1, "\r                                         \r" PROXPROMPT_COMPOSE, promptdev, promptnet, promptctx);
+        snprintf(buf, buflen - 1, "\33[2K\r" PROXPROMPT_COMPOSE, promptdev, promptnet, promptctx);
     }
 }
 
@@ -676,7 +656,7 @@ static void set_my_user_directory(void) {
         uint16_t pathLen = FILENAME_MAX; // should be a good starting point
         char *cwd_buffer = (char *)calloc(pathLen, sizeof(uint8_t));
         if (cwd_buffer == NULL) {
-            PrintAndLogEx(WARNING, "failed to allocate memory");
+            PrintAndLogEx(WARNING, "Failed to allocate memory");
             return;
         }
 
@@ -685,7 +665,7 @@ static void set_my_user_directory(void) {
                 pathLen += 10;      // if buffer was too small add 10 characters and try again
                 char *tmp = realloc(cwd_buffer, pathLen);
                 if (tmp == NULL) {
-                    PrintAndLogEx(WARNING, "failed to allocate memory");
+                    PrintAndLogEx(WARNING, "Failed to allocate memory");
                     free(cwd_buffer);
                     return;
                 }
@@ -780,7 +760,7 @@ static int dumpmem_to_file(const char *filename, uint32_t addr, uint32_t len, bo
 
     uint8_t *buffer = calloc(len, sizeof(uint8_t));
     if (buffer == NULL) {
-        PrintAndLogEx(ERR, "error, cannot allocate memory ");
+        PrintAndLogEx(WARNING, "Failed to allocate memory");
         return PM3_EMALLOC;
     }
 
@@ -1333,8 +1313,9 @@ int main(int argc, char *argv[]) {
     // This will allow the command line to override the settings.json values
     preferences_load();
     // quick patch for debug level
-    if (! debug_mode_forced)
+    if (! debug_mode_forced) {
         g_debugMode = g_session.client_debug_level;
+    }
     // settings_save ();
     // End Settings
 
