@@ -39,4 +39,9 @@ pacman -Ss '^gcc$'
 # sudo pacman -S testing/gcc
 # sudo pacman -S gcc
 
+python3 -m venv /tmp/venv
+source /tmp/venv/bin/activate
+python3 -m pip install --use-pep517 pyaes
+python3 -m pip install ansicolors sslcrypto
 tools/release_tests.sh
+deactivate
