@@ -48,7 +48,8 @@
 #define ICLASS_SIM_MODE_FULL                  3
 #define ICLASS_SIM_MODE_READER_ATTACK_KEYROLL 4
 #define ICLASS_SIM_MODE_EXIT_AFTER_MAC        5  // note: device internal only
-#define ICLASS_SIM_MODE_CONFIG_CARD           6
+#define ICLASS_SIM_MODE_FULL_GLITCH           6
+#define ICLASS_SIM_MODE_FULL_GLITCH_KEY       7
 
 
 // iCLASS auth request data structure
@@ -87,6 +88,17 @@ typedef struct {
     uint8_t mac[4];
 } PACKED iclass_writeblock_req_t;
 
+// iCLASS tearoff block request data structure
+typedef struct {
+    iclass_auth_req_t req;
+    uint8_t data[8];
+    uint8_t mac[4];
+    int tear_start;
+    int tear_end;
+    int increment;
+    int tear_loop;
+} PACKED iclass_tearblock_req_t;
+
 // iCLASS write block request data structure
 typedef struct {
     iclass_auth_req_t req;
@@ -108,6 +120,13 @@ typedef struct {
 typedef struct {
     iclass_auth_req_t req;
     iclass_auth_req_t req2;
+    uint32_t index;
+    uint32_t loop;
+    uint8_t nfa[8];
+    bool debug;
+    bool test;
+    bool fast;
+    bool short_delay;
 } PACKED iclass_recover_req_t;
 
 typedef struct iclass_premac {
@@ -149,12 +168,12 @@ typedef struct {
 
 // iCLASS secure mode memory mapping
 typedef struct {
-    uint8_t csn[8];
+    uint8_t csn[PICOPASS_BLOCK_SIZE];
     picopass_conf_block_t conf;
-    uint8_t epurse[8];
-    uint8_t key_d[8];
-    uint8_t key_c[8];
-    uint8_t app_issuer_area[8];
+    uint8_t epurse[PICOPASS_BLOCK_SIZE];
+    uint8_t key_d[PICOPASS_BLOCK_SIZE];
+    uint8_t key_c[PICOPASS_BLOCK_SIZE];
+    uint8_t app_issuer_area[PICOPASS_BLOCK_SIZE];
 } PACKED picopass_hdr_t;
 
 // iCLASS non-secure mode memory mapping
