@@ -84,8 +84,9 @@ typedef struct {
 typedef struct {
     uint32_t magic;
     uint16_t length : 15;  // length of the variable part, 0 if none.
-    bool ng : 1;
-    int16_t  status;
+    bool     ng : 1;
+    int8_t   status;
+    int8_t   reason;
     uint16_t cmd;
 } PACKED PacketResponseNGPreamble;
 
@@ -101,7 +102,8 @@ typedef struct {
     uint16_t cmd;
     uint16_t length;
     uint32_t magic;      //  NG
-    int16_t  status;     //  NG
+    int8_t   status;     //  NG
+    int8_t   reason;     //  NG
     uint16_t crc;        //  NG
     uint64_t oldarg[3];  //  OLD
     union {
@@ -129,6 +131,27 @@ typedef struct {
     bool verbose;
 } PACKED sample_config;
 
+
+// Defines a frame that will be used in a polling sequence
+// Polling loop annotations are up to 20 bytes long, 24 bytes should cover future and other cases
+typedef struct {
+    uint8_t frame[24];
+    // negative values can be used to carry special info
+    int8_t frame_length;
+    uint8_t last_byte_bits;
+    uint16_t extra_delay;
+} PACKED iso14a_polling_frame_t;
+
+
+// Defines polling sequence configuration
+typedef struct {
+    // 6 would be enough for 4 magsafe, 1 wupa, 1 pla,
+    iso14a_polling_frame_t frames[6];
+    int8_t frame_count;
+    uint16_t extra_timeout;
+} PACKED iso14a_polling_parameters_t;
+
+
 // A struct used to send hf14a-configs over USB
 typedef struct {
     int8_t forceanticol; // 0:auto 1:force executing anticol 2:force skipping anticol
@@ -136,7 +159,9 @@ typedef struct {
     int8_t forcecl2;     // 0:auto 1:force executing CL2 2:force skipping CL2
     int8_t forcecl3;     // 0:auto 1:force executing CL3 2:force skipping CL3
     int8_t forcerats;    // 0:auto 1:force executing RATS 2:force skipping RATS
-} PACKED hf14a_config;
+    int8_t magsafe;      // 0:disabled 1:enabled
+    iso14a_polling_frame_t polling_loop_annotation; // Polling loop annotation
+} PACKED hf14a_config_t;
 
 // Tracelog Header struct
 typedef struct {
@@ -321,6 +346,7 @@ typedef struct {
 typedef struct {
     uint8_t sectorcnt;
     uint8_t keytype;
+    uint8_t key[6];
 } PACKED mfc_eload_t;
 
 typedef struct {
@@ -430,6 +456,7 @@ typedef struct {
 #define CMD_FLASHMEM_DOWNLOADED                                           0x0124
 #define CMD_FLASHMEM_INFO                                                 0x0125
 #define CMD_FLASHMEM_SET_SPIBAUDRATE                                      0x0126
+#define CMD_FLASHMEM_PAGES64K                                             0x0127
 
 // RDV40, High level flashmem SPIFFS Manipulation
 // ALL function will have a lazy or Safe version
@@ -571,6 +598,7 @@ typedef struct {
 #define CMD_HF_ISO15693_SLIX_PASS_PROTECT_AFI                             0x0863
 #define CMD_HF_ISO15693_SLIX_PASS_PROTECT_EAS                             0x0864
 #define CMD_HF_ISO15693_SLIX_WRITE_PWD                                    0x0865
+#define CMD_HF_ISO15693_SLIX_PROTECT_PAGE                                 0x0868
 #define CMD_HF_ISO15693_WRITE_AFI                                         0x0866
 #define CMD_HF_TEXKOM_SIMULATE                                            0x0320
 #define CMD_HF_ISO15693_EML_CLEAR                                         0x0330
@@ -595,6 +623,13 @@ typedef struct {
 #define CMD_LF_HITAGS_SIMULATE                                            0x0368
 #define CMD_LF_HITAGS_READ                                                0x0373
 #define CMD_LF_HITAGS_WRITE                                               0x0375
+#define CMD_LF_HITAGS_UID                                                 0x037A
+
+// For Hitag µ
+#define CMD_LF_HITAGU_READ                                                0x037B
+#define CMD_LF_HITAGU_WRITE                                               0x037C
+#define CMD_LF_HITAGU_SIMULATE                                            0x037D
+#define CMD_LF_HITAGU_UID                                                 0x037E
 
 #define CMD_LF_HITAG_ELOAD                                                0x0376
 
@@ -604,8 +639,10 @@ typedef struct {
 
 #define CMD_HF_ISO14443A_SNIFF                                            0x0383
 #define CMD_HF_ISO14443A_SIMULATE                                         0x0384
+#define CMD_HF_ISO14443A_SIM_AID                                          0x1420
 
 #define CMD_HF_ISO14443A_READER                                           0x0385
+#define CMD_HF_ISO14443A_EMV_SIMULATE                                     0x0386
 
 #define CMD_HF_LEGIC_SIMULATE                                             0x0387
 #define CMD_HF_LEGIC_READER                                               0x0388
@@ -631,6 +668,7 @@ typedef struct {
 #define CMD_HF_ICLASS_RESTORE                                             0x039B
 #define CMD_HF_ICLASS_CREDIT_EPURSE                                       0x039C
 #define CMD_HF_ICLASS_RECOVER                                             0x039D
+#define CMD_HF_ICLASS_TEARBL                                              0x039E
 
 
 // For ISO1092 / FeliCa
@@ -761,19 +799,73 @@ typedef struct {
 #define CMD_UNKNOWN                                                       0xFFFF
 
 //Mifare simulation flags
-#define FLAG_INTERACTIVE        0x01
-#define FLAG_4B_UID_IN_DATA     0x02
-#define FLAG_7B_UID_IN_DATA     0x04
-#define FLAG_10B_UID_IN_DATA    0x08
-#define FLAG_UID_IN_EMUL        0x10
-#define FLAG_NR_AR_ATTACK       0x20
-#define FLAG_MF_MINI            0x80
-#define FLAG_MF_1K              0x100
-#define FLAG_MF_2K              0x200
-#define FLAG_MF_4K              0x400
-#define FLAG_FORCED_ATQA        0x800
-#define FLAG_FORCED_SAK         0x1000
-#define FLAG_CVE21_0430         0x2000
+// In interactive mode, we are expected to finish the operation with an ACK
+#define FLAG_INTERACTIVE        0x0001
+#define FLAG_ATQA_IN_DATA       0x0002
+#define FLAG_SAK_IN_DATA        0x0004
+#define FLAG_ATS_IN_DATA        0x0008
+#define FLAG_ENUMERATE_AID      0x0010
+
+// internal constants, use the function macros instead
+#define FLAG_MASK_UID           0x0030
+#define FLAG_UID_IN_EMUL        0x0000
+#define FLAG_4B_UID_IN_DATA     0x0010
+#define FLAG_7B_UID_IN_DATA     0x0020
+#define FLAG_10B_UID_IN_DATA    0x0030
+// if there is a UID in the data-section to be used:
+// note: if UIDLEN is wrong, we default to FLAG_UID_IN_EMUL
+#define FLAG_SET_UID_IN_DATA(flags, len) {\
+    flags = (flags & (~FLAG_MASK_UID))|\
+        (len == 4 ? FLAG_4B_UID_IN_DATA : \
+        (len == 7 ? FLAG_7B_UID_IN_DATA : \
+        (len == 10 ? FLAG_10B_UID_IN_DATA : \
+        FLAG_UID_IN_EMUL)));\
+    }
+// else we tell to take UID from block 0:
+#define FLAG_SET_UID_IN_EMUL(flags) {flags = (flags & (~FLAG_MASK_UID))|FLAG_UID_IN_EMUL;}
+#define IS_FLAG_UID_IN_DATA(flags, len) (\
+    (flags & FLAG_MASK_UID) == \
+        (len == 4 ? FLAG_4B_UID_IN_DATA : \
+        (len == 7 ? FLAG_7B_UID_IN_DATA : \
+        (len == 10 ? FLAG_10B_UID_IN_DATA : \
+        FLAG_UID_IN_EMUL)))\
+    )
+#define IS_FLAG_UID_IN_EMUL(flags) ((flags & FLAG_MASK_UID) == FLAG_UID_IN_EMUL)
+
+// internal constants, use the function macros instead
+#define MIFARE_4K_MAX_BYTES     4096
+#define MIFARE_2K_MAX_BYTES     2048
+#define MIFARE_1K_MAX_BYTES     1024
+#define MIFARE_MINI_MAX_BYTES   320
+#define FLAG_MASK_MF_SIZE       0x00C0
+#define FLAG_MF_MINI            0x0000
+#define FLAG_MF_1K              0x0040
+#define FLAG_MF_2K              0x0080
+#define FLAG_MF_4K              0x00C0
+#define FLAG_SET_MF_SIZE(flags, size) {\
+    flags = (flags & (~FLAG_MASK_MF_SIZE))|\
+        (size == MIFARE_MINI_MAX_BYTES ? FLAG_MF_MINI : \
+        (size == MIFARE_1K_MAX_BYTES ? FLAG_MF_1K : \
+        (size == MIFARE_2K_MAX_BYTES ? FLAG_MF_2K : \
+        (size == MIFARE_4K_MAX_BYTES ? FLAG_MF_4K : \
+        0))));\
+    }
+// else we tell to take UID from block 0:
+#define IS_FLAG_MF_SIZE(flags, size) (\
+    (flags & FLAG_MASK_MF_SIZE) == \
+        (size == MIFARE_MINI_MAX_BYTES ? FLAG_MF_MINI : \
+        (size == MIFARE_1K_MAX_BYTES ? FLAG_MF_1K : \
+        (size == MIFARE_2K_MAX_BYTES ? FLAG_MF_2K : \
+        (size == MIFARE_4K_MAX_BYTES ? FLAG_MF_4K : \
+        0))))\
+    )
+
+#define FLAG_MF_USE_READ_KEYB   0x0100
+#define FLAG_CVE21_0430         0x0200
+// collect NR_AR responses for bruteforcing later
+#define FLAG_NR_AR_ATTACK       0x0400
+// support nested authentication attack
+#define FLAG_NESTED_AUTH_ATTACK 0x0800
 
 
 #define MODE_SIM_CSN        0
@@ -781,10 +873,11 @@ typedef struct {
 #define MODE_FULLSIM        2
 
 // Static Nonce detection
-#define NONCE_FAIL       0x01
-#define NONCE_NORMAL     0x02
-#define NONCE_STATIC     0x03
-#define NONCE_STATIC_ENC 0x04
+#define NONCE_FAIL        0x01
+#define NONCE_NORMAL      0x02
+#define NONCE_STATIC      0x03
+#define NONCE_STATIC_ENC  0x04
+#define NONCE_SUPERSTATIC 0x05
 
 // Dbprintf flags
 #define FLAG_RAWPRINT    0x00
@@ -859,13 +952,24 @@ typedef struct {
 // Got wrong length error               pm3: when received wrong length of data
 #define PM3_ELENGTH           -27
 
-// No data                              pm3:        no data available, no host frame available (not really an error)
+// No key available                     client/pm3: no cryptographic key available.
+#define PM3_ENOKEY            -28
+
+// Cryptographic error                  client/pm3: cryptographic operation failed
+#define PM3_ECRYPTO           -29
+
+// No data                              client/pm3: no data available, no host frame available (not really an error)
 #define PM3_ENODATA           -98
 // Quit program                         client:     reserved, order to quit the program
 #define PM3_EFATAL            -99
 // Regular quit
 #define PM3_SQUIT            -100
 
+// reserved for future protocol change
+#define PM3_RESERVED         -128
+
+#define PM3_REASON_UNKNOWN     -1
+
 // LF
 #define LF_FREQ2DIV(f) ((int)(((12000.0 + (f)/2.0)/(f))-1))
 #define LF_DIVISOR_125 LF_FREQ2DIV(125)
